 /*
 * Copyright 
 *   2008 axYus - www.axyus.com
 *   2008 jp.tessier - jean-philippe.tessier@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.plugin.generatorEdmn;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import fr.gouv.finances.cp.utils.ui.DisplayExceptionDlg;
import fr.gouv.finances.cp.xemelios.data.impl.MySqlDataLayer;
import fr.gouv.finances.cp.xemelios.data.DataConfigurationException;
import fr.gouv.finances.cp.xemelios.data.DataLayerManager;
import fr.gouv.finances.cp.xemelios.importers.DefaultImporter;
import java.lang.reflect.Constructor;

/**
 *
 * @author chm
 */
public class EdmnImporter extends DefaultImporter {

    private static final Logger logger = Logger.getLogger(EdmnImporter.class);
    private static String TABLE_INDEX = "EDMN_INDEX_NUM_LIASSE";
    // Variables de progression d'avancement de la progrsse barre
    private static ProgressFrame progress;
    private DlgGenEdmn dlg;

    public EdmnImporter() {
        super();
    }

    public EdmnImporter(DlgGenEdmn dlg) {
        super();
        this.dlg = dlg;
    }

    /**
     * @author jp.tessier
     * @param f : file EDMN
     * @param index : index of correspondance NumLiasseLocale <=> NumLiasseNationnale
     * @throws Exception
     */
    public void importFile(File f, File index) throws Exception {
        Connection con = null;
        try {
            MySqlDataLayer msdl = (MySqlDataLayer) DataLayerManager.getImplementation();
            con = msdl.getConnection();
            createDataBase(con);

            Class<EdmnImporteurWorker> clazz = EdmnImporteurWorker.class;
            Constructor<EdmnImporteurWorker> cc = clazz.getConstructor(Connection.class);
            EdmnImporteurWorker eiw = cc.newInstance(con);

            File[] tabF = {f, index};
            progress = new ProgressFrame(dlg, eiw, tabF);
            progress.setDisplayFeedback(true);

            eiw.setProgressDlg(progress);
            eiw.setFilesToTreat(tabF);
            eiw.execute();
        } catch (Exception ex) {
            ex.printStackTrace();
            logger.error("doImport", ex);
            new DisplayExceptionDlg(dlg, ex);
        } finally {
            // con is closed by EdmnImporterWorker
//            if (con != null) {
//                con.close();
//            }
            if(progress!=null && progress.pg!=null)
                progress.pg.setVisible(false);
        }
    }
        

    private void createDataBase(Connection con) throws SQLException {
        StringBuilder sb = new StringBuilder();
        PreparedStatement ps = null;
        try {
            // Creation de la table si inexistente
            sb.append("CREATE TEMPORARY TABLE IF NOT EXISTS " + TABLE_INDEX + " ( ");
            sb.append("ANNEE VARCHAR(2) NOT NULL, ");
            sb.append("TRIMESTRE VARCHAR(1) NOT NULL, ");
            sb.append("COMPTABLE VARCHAR(6) NOT NULL, ");
            sb.append("MINISTERE VARCHAR(3) NOT NULL, ");
            sb.append("LOCALE VARCHAR(4) NOT NULL, ");
            sb.append("GESTION VARCHAR(3) NOT NULL, ");
            sb.append("SECTION VARCHAR(3) NOT NULL, ");
            sb.append("NATIONALE VARCHAR(6) NOT NULL, ");
            sb.append("INDEX toto (ANNEE,TRIMESTRE,COMPTABLE,MINISTERE,LOCALE)");
            sb.append(")");
            ps = con.prepareStatement(sb.toString());
            ps.execute();
            // Vidage de la table d'index
            sb = new StringBuilder();
            sb.append("TRUNCATE TABLE " + TABLE_INDEX);
            ps = con.prepareStatement(sb.toString());
            ps.execute();
        } catch (SQLException sqlEx) {
            sqlEx.printStackTrace();
        } finally {
            if (ps != null) {
                try {
                    ps.close();
                } catch (Throwable t) {
                }
            }
        }
    }
    
}
