/*
 * Copyright 
 *   2008 axYus - www.axyus.com
 *   2008 jp.tessier - jean-philippe.tessier@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.plugin.generatorEdmn;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.zip.Adler32;
import java.util.zip.CheckedOutputStream;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.jdesktop.swingworker.SwingWorker;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;

import fr.gouv.finances.cp.xemelios.ui.MainWindow;
import fr.gouv.finances.cp.xemelios.ui.MainWindow.ImportContent;
import fr.gouv.finances.cp.xemelios.common.FileInfo;
import fr.gouv.finances.cp.xemelios.common.PJRef;
import fr.gouv.finances.cp.xemelios.common.config.DocumentModel;
import fr.gouv.finances.cp.xemelios.importers.EtatImporteur;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * Classe a surcharger pour creer des importeurs 
 * @author chm
 */
public class EdmnImporteurWorker extends SwingWorker<String, String> {

    private static Logger logger = Logger.getLogger(EdmnImporteurWorker.class);
    protected File[] filesToTreat;
    protected Exception inProcessException = null;
    protected long start;
    public final int MSG_TYPE_INFORMATION = 1;
    public final int MSG_TYPE_ERROR = 2;
    private static String TABLE_INDEX = "EDMN_INDEX_NUM_LIASSE";
    public static final String EDMN_NS_URI = "http://projets.admisource.gouv.fr/xemelios/namespaces#edmn";
    private ProgressFrame progress;
    private int progressShowing = 0;
    private String fireName = "";
    private Object fireValue;
    private File dirEDMN=null;
    /** Taille du buffer pour les lectures/critures */
	private static final int BUFFER_SIZE = 8 * 1024;

    private Connection con = null;
	
    public EdmnImporteurWorker(Connection con) {
        super();
        this.con=con;
    }

    @Override
    public String doInBackground() {
        FileInfo fileInfo = new FileInfo();
        start = System.currentTimeMillis();
        this.getProgressDlg().displayProgress();
        try {
            try {
                File f = filesToTreat[1];
                final String fName = f.getName();

                // Sauvegarde de l'index en base
                fireProperty("FILE_NAME", fName);
                fireProperty("initialize", getProgressDlg().nbLines[1]);
                saveIndex(f, getProgressDlg().nbLines[1]);
                fireProperty("currentTerminated", null);
            } catch (Exception ex) {
                inProcessException = ex;
                ex.printStackTrace();
            }
            try {
                File f = filesToTreat[0];
                final String fName = f.getName();

                // Cration du fichier EDMN
                fireProperty("FILE_NAME", fName);
                fireProperty("initialize", getProgressDlg().nbLines[0]);
                importFile(f, getProgressDlg().nbLines[0]);
                fireProperty("currentTerminated", null);
            } catch (Exception ex) {
                inProcessException = ex;
                ex.printStackTrace();
            }
        } catch (Exception ex) {
            inProcessException = ex;
            ex.printStackTrace();
        } finally {
            if(con!=null) {
                try {
                    con.close();
                } catch(SQLException sqlEx) {
                    // nothing to do
                }
            }
        }
//        try {
//            postProcess(inProcessException != null);
//        } catch (Exception ex) {
//            inProcessException = ex;
//            ex.printStackTrace();
//        }
        StringBuffer msg = new StringBuffer();
        if (!isCancelled()) {
            long ms = (System.currentTimeMillis() - start);
            int s = Integer.parseInt(String.valueOf(ms / 1000));
            int m = Integer.parseInt(String.valueOf(s / 60));
            int h = Integer.parseInt(String.valueOf(m / 60));


            StringBuffer sb = new StringBuffer("La gnration a dur : ");
            if (Integer.parseInt(String.valueOf(h)) != 0) {
                sb.append(h + "h ");
            }
            if (Integer.parseInt(String.valueOf(m)) != 0) {
                sb.append((h != 0 ? " et " : "") + (m - (h * 60)) + "mins ");
            }
            if (Integer.parseInt(String.valueOf(s)) != 0) {
                sb.append(((h != 0 || m != 0) ? " et " : "") + (s - (m * 60)) + "s ");
            }
            if (Integer.parseInt(String.valueOf(ms)) != 0) {
                sb.append(((h != 0 || m != 0 || s != 0) ? " et " : "") + (ms - (s * 1000)) + "ms ");
            }

            msg.append("La gnration du fichier est termin.\n"/* + sb.toString()*/);
            if (false/*fileInfo.getGlobalCount() == 0*/) {
                msg.append("Pas de donnes gnres.");
            } else {
                msg.append(fileInfo.toString());
                msg.append("");
            }
        } else {
            msg.append("La gnration du fichier EDMN a t interrompue.");
        }
        logger.info("\n" + msg);
//        fireProperty("terminated", msg);	// histoire de pas faire un new
        return msg.toString();
    }

    private void fireProperty(String name, Object value) {
        fireName = name;
        fireValue = value;
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                firePropertyChange(name, null, value);
            } else {
                SwingUtilities.invokeAndWait(new Runnable() {

                    public void run() {
                        firePropertyChange(fireName, null, fireValue);
                    }
                });
            }
        } catch (Throwable t) {
            logger.fatal("in fire:", t);
        }
    }

    @Override
    protected void done() {
        super.done();
        if (inProcessException != null) {
            getProgressDlg().displayException(inProcessException);
        }
        String msg = "";
        try {
            msg = get();
        } catch (InterruptedException iEx) {
            iEx.printStackTrace();
        } catch (ExecutionException eEx) {
            eEx.printStackTrace();
        } catch (CancellationException cEx) {
            msg = "Gnration annule";
        }
        if(inProcessException==null)
            importEdmnFile();
        else
            getProgressDlg().displayMessage(msg, MSG_TYPE_INFORMATION);
    }

    public void setFilesToTreat(File[] fichiers) throws Exception {
        filesToTreat = fichiers;
    }


    public ProgressFrame getProgressDlg() {
        return progress;
    }

    public void setProgressDlg(ProgressFrame prog) {
        this.progress = prog;
    }

    protected void saveIndex(File index, int nbLignes) throws Exception {
        BufferedReader br;
        String line;
        PreparedStatement ps = null;
        StringBuffer sb = new StringBuffer();
//        Connection con = null;
        File fSQL = new File(System.getProperty("user.home"), "insert_index.txt");
        FileWriter fw = new FileWriter(fSQL);
        progressShowing = (int) nbLignes / 500;

        try {
            // Cration de la connection
//            MySqlDataLayer msdl = (MySqlDataLayer) DataLayerManager.getImplementation();
//            con = msdl.getConnection();

            br = new BufferedReader(new FileReader(index));
            line = br.readLine();

            int i = 0;
            int total = 0;
            Long tempsTotalTraitement = Long.parseLong("0");
            Date datedeb = new Date();
            while (line != null) {
                i++;
                total++;
                if (line.length() > 15) {
                    // Dernire ligne?
                    if (total == nbLignes) {
                        sb.append(line.substring(0, 2) + "," + line.substring(2, 3) + "," + line.substring(3, 9) + "," + line.substring(9, 12) + "," + line.substring(17, 21) + "," + line.substring(21, 24) + "," + line.substring(24, 26) + "," + line.substring(31, 37));
                    } else {
                        sb.append(line.substring(0, 2) + "," + line.substring(2, 3) + "," + line.substring(3, 9) + "," + line.substring(9, 12) + "," + line.substring(17, 21) + "," + line.substring(21, 24) + "," + line.substring(24, 26) + "," + line.substring(31, 37) + "\n");
                    }
                }
                if (i == progressShowing) {
                    fireProperty("progress", i);
                    i = 0;
                }
                line = br.readLine();
            }
            if (i != 0) {
                fireProperty("progress", i);
            }
            i = 0;
            fw.write(sb.toString());
            fw.flush();
            fw.close();
            // Insertion en base des valeurs
            ps = con.prepareStatement("LOAD DATA LOCAL INFILE '" + fSQL.getAbsolutePath().replaceAll("\\\\", "\\\\\\\\") + "' INTO TABLE " + TABLE_INDEX + " FIELDS TERMINATED BY ','");
            ps.execute();
            fSQL.delete();

            // Calcul de temps de traitement
            Date datefin = new Date();
            tempsTotalTraitement += (datefin.getTime() - datedeb.getTime());
            int minutes = Integer.parseInt(Long.toString(tempsTotalTraitement / (60 * 1000)));
            int secondes = Integer.parseInt(Long.toString(tempsTotalTraitement / (1000)));
            int millis = Integer.parseInt(Long.toString(tempsTotalTraitement));

            logger.info("Temps d'insertion = " + ((minutes != 0) ? minutes + "mins et " : "") + ((secondes) - (minutes * 60)) + "s et " + ((millis) - (secondes * 1000)) + "ms");

        } catch (IOException e) {
            if (e.getClass().equals(FileNotFoundException.class)) {
                logger.debug("Le fichier d'index n'a pas t trouv !", e.getCause());
            } else {
                logger.debug("IOException sur le fichier " + index.getAbsolutePath() + " !", e.getCause());
            }
        } catch (SQLException e) {
            throw e;
//        } catch (DataConfigurationException e) {
//            throw e;
        } finally {
            if (ps != null) {
                try {
                    ps.close();
                } catch (Throwable t) {
                }
            }
//            if (con != null) {
//                con.close();
//            }
        }
    }

    protected FileInfo importFile(File f, int nbLignes) throws Exception {
        FileInfo ret = new FileInfo();
        
        dirEDMN = new File(f.getParent(),"CG");
        dirEDMN.mkdir();
        if(dirEDMN!=null && dirEDMN.listFiles().length>0){
        	for(File fToDelete:dirEDMN.listFiles()){
        		fToDelete.delete();
        	}
        }
        
        generateXmlFileText(f,dirEDMN,nbLignes);
        return ret;
    }

    protected String getNumLiasseNationale(String annee, String trimestre, String comptable, String ministere, String numLiasseLocale, Connection con) {

        PreparedStatement ps = null;
        String ret = "";
        try {
            // Cration d'un prepared statement
            ps = con.prepareStatement("SELECT NATIONALE FROM " + TABLE_INDEX + " WHERE ANNEE = ? AND TRIMESTRE = ? AND COMPTABLE = ? AND MINISTERE = ? AND LOCALE = ?");
            ps.setString(1, annee);
            ps.setString(2, trimestre);
            ps.setString(3, comptable);
            ps.setString(4, ministere);
            ps.setString(5, numLiasseLocale);

            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                ret = rs.getString(1);
                if (rs.next()) {
                    ret = "MULTI";
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            if (ps != null) {
                try {
                    ps.close();
                } catch (Throwable t) {
                }
            }
        }
        return ret;
    }

//    private Connection getConnection() {
//        MySqlDataLayer msdl;
//        Connection ret = null;
//        try {
//            msdl = (MySqlDataLayer) DataLayerManager.getImplementation();
//            ret = msdl.getConnection();
//        } catch (SQLException e) {
//            e.printStackTrace();
//        } catch (DataConfigurationException e) {
//            e.printStackTrace();
//        }
//        return ret;
//    }

    /*protected void generateXmlFile(File source, File dest, int nbLignes) throws IOException, SAXException, SQLException {
        progressShowing = nbLignes / 500;
        OutputStream os = new FileOutputStream(dest);
        SAXWriter sWriter = new SAXWriter(os, "ISO-8859-1");
        BufferedReader br = new BufferedReader(new FileReader(source));
        String line = br.readLine();
        Connection con = getConnection();

        sWriter.startDocument();
        sWriter.startPrefixMapping("n", EDMN_NS_URI);
        sWriter.startElement(EDMN_NS_URI, "Edmn", "n:Edmn", new AttributesImpl());
        sWriter.startElement(EDMN_NS_URI, "EtatEdmn", "n:EtatEdmn", new AttributesImpl());

        int i = 0;
        try {
            while (line != null) {
                i++;
                createLine(line.toCharArray(), sWriter, con);
                if (i == progressShowing) {
                    fireProperty("progress", i);
                }
                i = 0;
                line = br.readLine();
            }
        } catch (SAXException e) {
            logger.debug("", e);
        }
        if (i != 0) {
            fireProperty("progress", i);
        }
        if (con != null) {
            con.close();
        }
        sWriter.endPrefixMapping("n");
        sWriter.endElement(EDMN_NS_URI, "EtatEdmn", "n:EtatEdmn");
        sWriter.endElement(EDMN_NS_URI, "Edmn", "n:Edmn");
        sWriter.endDocument();
        os.flush();
        os.close();
    }*/

    protected void generateXmlFileText(File source, File dirDest, int nbLignes) throws IOException, SAXException, SQLException {
        progressShowing = nbLignes / (nbLignes / 500);
        
        dirDest.mkdir();
        
        FileOutputStream fWriter=null;
        BufferedReader br = new BufferedReader(new FileReader(source));
       
//        Connection con = getConnection();
        
        int breakFile=10000;
        int nbLignesToSplitFile = (int)Math.ceil(Double.parseDouble(String.valueOf(nbLignes))/breakFile);
        int nbLignesParcourues = 0;
        int i = 0;
        int iLines = 0;
        Vector<String> vLignes = new Vector<String>();
        long start = 0L;
        long startLines = 0L;
        int nbLinesToGroup = 500;
    	String line = br.readLine();
        for(int numFile = 1;numFile<=nbLignesToSplitFile;numFile++){
        	fWriter = new FileOutputStream(new File(dirDest.getAbsolutePath(),source.getName()+"-"+numFile+".xml"));        	
        	fWriter.write(("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<n:Edmn xmlns=\"" + EDMN_NS_URI + "\" xmlns:n=\"" + EDMN_NS_URI + "\">\n<EtatEdmn>\n").getBytes());
        	fWriter.write("<Budget><Code V=\"0000\"/><Libelle V=\"LFI\"/></Budget>\n".getBytes());
        	fWriter.write("<Collectivite><Code V=\"000000000000\"/><Libelle V=\"ETAT\"/></Collectivite>\n".getBytes());
        	try {
                while (line != null && nbLignesParcourues!=breakFile) {
                	nbLignesParcourues++;
                    if (iLines == 0) {
                        startLines = System.currentTimeMillis();
                    }
                    i++;
                    iLines++;
                    vLignes.add(createLineText(line.toCharArray(), con));
                    if (vLignes.size() == nbLinesToGroup) {
                        iLines = 0;
                        //logger.debug("create "+vLignes.size()+" lines in "+((System.currentTimeMillis()-startLines)+"ms"));
                        start = System.currentTimeMillis();
                        StringBuilder sb = new StringBuilder();
                        for (String s : vLignes) {
                            sb.append(s);
                        }
                        fWriter.write(sb.toString().getBytes());
                        //logger.debug("insert "+vLignes.size()+" lines in "+((System.currentTimeMillis()-start)+"ms"));
                        vLignes.clear();
                    }
                    if (i == progressShowing) {
                        fireProperty("progress", i);
                        i = 0;
                    }
                    line = br.readLine();
                }
            } catch (SAXException e) {
                logger.debug("", e);
            }
            nbLignesParcourues=0;
            if (vLignes.size() > 0) {
                StringBuilder sb = new StringBuilder();
                for (String s : vLignes) {
                    sb.append(s);
                }
                fWriter.write(sb.toString().getBytes());
                vLignes.clear();
            }
        	fWriter.write("</EtatEdmn>\n</n:Edmn>".getBytes());
            fWriter.flush();
            fWriter.close();
        }
        if (i != 0) {fireProperty("progress", i);}
//        if (con != null) {con.close();}
    }

    public String getAdditionalMessage() {
        return "";
    }

   /* private void createLine(char[] line, DefaultHandler h, Connection con) throws SAXException {
        h.startElement(EDMN_NS_URI, "LigneEdmn", "n:LigneEdmn", new AttributesImpl());
        writeNode("Comptable", getLine(line, 0, 6), h);
        writeNode("Ministere", getLine(line, 6, 9), h);
        writeNode("Ordo", getLine(line, 9, 15), h);
        writeNode("TypeEvt", getLine(line, 15, 18), h);
        writeNode("AnneeBudget", getLine(line, 18, 20), h);
        writeNode("NumMandat", getLine(line, 20, 26), h);
        writeNode("CompteSpeTresor", getLine(line, 26, 31), h);
        writeNode("NumOpeInvest", getLine(line, 31, 39), h);
        writeNode("AnneeEngag", getLine(line, 39, 41), h);
        writeNode("NumEngag", getLine(line, 41, 47), h);
        writeNode("ChapBudg", getLine(line, 47, 51), h);
        writeNode("ArtBudgExec", getLine(line, 51, 53), h);
        writeNode("ParagBudgExec", getLine(line, 53, 55), h);
        String mtedm = getLine(line, 55, 68);
        while (mtedm.startsWith("0")) {
            mtedm = mtedm.substring(1);
        }
        if (getLine(line, 68, 69).trim().equals("-")) {
            mtedm = "-" + mtedm;
        }
        writeNode("MtPJ", mtedm, h);
        writeNode("Trimestre", getLine(line, 69, 70), h);
        writeNode("Mois", getLine(line, 70, 72), h);
        writeNode("AnneeCrea", getLine(line, 72, 76), h);
        writeNode("MoisCrea", getLine(line, 76, 78), h);
        writeNode("JourCrea", getLine(line, 78, 80), h);
        writeNode("ChapClassPJ", getLine(line, 80, 84), h);
        writeNode("TypePaiement", getLine(line, 84, 87), h);
        writeNode("ArtBudgPrev", getLine(line, 87, 89), h);
        writeNode("Zone24", getLine(line, 89, 113), h);
        writeNode("NumLiasse", getLine(line, 113, 117) + getLine(line, 117, 121), h);
        writeNode("TypeSuiviEco1", getLine(line, 121, 123), h);
        writeNode("TypeSuiviEco2", getLine(line, 123, 125), h);
        writeNode("TypeSuiviEco3", getLine(line, 125, 127), h);
        writeNode("CodeCorrection", getLine(line, 127, 131), h);
        //writeNode("NumLiasseNationale",getNumLiasseNationale(getLine(line,131,137),con),h);
        writeNode("CodeManip", getLine(line, 137, 138), h);
        writeNode("Zome32", getLine(line, 138, 170), h);
        h.endElement(EDMN_NS_URI, "LigneEdmn", "n:LigneEdmn");
    }*/

    private String getLine(char[] tab, int deb, int fin) {
        String s = "";
        for (int i = deb; i < fin; i++) {
            s += String.valueOf(tab[i]);
        }
        return s.trim();
    }

    protected void writeNode(String nodeName, String value, DefaultHandler h) throws SAXException {
        AttributesImpl attr = new AttributesImpl();
        attr.addAttribute("", "V", "V", "String", value);
        h.startElement(EDMN_NS_URI, nodeName, "n:" + nodeName, attr);
        h.endElement(EDMN_NS_URI, nodeName, "n:" + nodeName);
    }

    private String createLineText(char[] line, Connection con) throws SAXException {
        StringBuilder sb = new StringBuilder("<LigneEdmn>");
        sb.append("<Comptable V=\"" + getLine(line, 0, 6) + "\"/><Ministere V=\"" + getLine(line, 6, 9) + "\"/><Ordo V=\"" + getLine(line, 9, 15) + "\"/><TypeEvt V=\"" + getLine(line, 15, 18) + "\"/><AnneeBudget V=\"" + getLine(line, 18, 20) + "\"/><NumMandat V=\"" + getLine(line, 20, 26) + "\"/><CompteSpeTresor V=\"" + getLine(line, 26, 31) + "\"/><NumOpeInvest V=\"" + getLine(line, 31, 39) + "\"/><AnneeEngag V=\"" + getLine(line, 39, 41) + "\"/><NumEngag V=\"" + getLine(line, 41, 47) + "\"/><ChapBudg V=\"" + getLine(line, 47, 51) + "\"/><ArtBudgExec V=\"" + getLine(line, 51, 53) + "\"/><ParagBudgExec V=\"" + getLine(line, 53, 55) + "\"/>");
        String mtedm = getLine(line, 55, 68);
        while (mtedm.startsWith("<0")) {
            mtedm = mtedm.substring(1);
        }
        if (getLine(line, 68, 69).trim().equals("-")) {
            mtedm = "-" + mtedm;
        }
        return sb.append("<MtPJ V=\"" + mtedm + "\"/><Trimestre V=\"" + getLine(line, 69, 70) + "\"/><Mois V=\"" + getLine(line, 70, 72) + "\"/><AnneeCrea V=\"" + getLine(line, 72, 76) + "\"/><MoisCrea V=\"" + getLine(line, 76, 78) + "\"/><JourCrea V=\"" + getLine(line, 78, 80) + "\"/><ChapClassPJ V=\"" + getLine(line, 80, 84) + "\"/><TypePaiement V=\"" + getLine(line, 84, 87) + "\"/><ArtBudgPrev V=\"" + getLine(line, 87, 89) + "\"/><Zone24 V=\"" + getLine(line, 89, 113) + "\"/><NumLiasse V=\"" + getLine(line, 113, 117) + getLine(line, 117, 121) + "\"/><TypeSuiviEco1 V=\"" + getLine(line, 121, 123) + "\"/><TypeSuiviEco2 V=\"" + getLine(line, 123, 125) + "\"/><TypeSuiviEco3 V=\"" + getLine(line, 125, 127) + "\"/><CodeCorrection V=\"" + getLine(line, 127, 131) + "\"/><NumLiasseNationale V=\"" + getNumLiasseNationale(getLine(line, 74, 76), getLine(line, 69, 70), getLine(line, 0, 6), getLine(line, 6, 9), getLine(line, 117, 121), con) + "\"/><CodeManip V=\"" + getLine(line, 137, 138) + "\"/><Zome32 V=\"" + getLine(line, 138, 170) + "\"/></LigneEdmn>\n").toString();
    }    

    private void importEdmnFile(){
    	if(MainWindow.instance!=null) {
    		//int reponse = JOptionPane.showConfirmDialog(MainWindow.instance,"Souhaitez-vous importer le fichier gnr ?","Question",JOptionPane.YES_NO_OPTION);
    		//if(JOptionPane.YES_OPTION == reponse){
    			DocumentModel dm = MainWindow.instance.getDocModels().getDocumentById("edmn");
    			try {
    				Class clazz = Class.forName(dm.getImportClass());
    		        Object obj = clazz.newInstance();
    				if(!(obj instanceof EtatImporteur)) {
    					JOptionPane.showMessageDialog(MainWindow.instance,"Cette classe n'est pas un importeur.\nLe fichier de configuration qui vous a t livr est certainement invalide.\nVeuillez contacter votre fournisseur.\n","Erreur",JOptionPane.ERROR_MESSAGE);
    					return;
    				}
    				EtatImporteur ei = (EtatImporteur)obj;
    				ei.setDocument(dm);
//    				ArrayList<File> array = new ArrayList<File>();
    				/* Zippage avant import */
// TODO: est-ce bien necessaire ? pour le transport, c'est pas mal...
        			File zip = makeZip();

                    MainWindow.ImportContent ic = new MainWindow.ImportContent();
                    File tmpDir = new File(System.getProperty("java.io.tmpdir"));

                    ZipFile zf = null;
                    try {
                        zf = new ZipFile(zip);
                        for (Enumeration<? extends ZipEntry> enumer = zf.entries(); enumer.hasMoreElements();) {
                            ZipEntry ze = enumer.nextElement();
                            if (!ze.isDirectory()) {
                                String fileName = ze.getName();
                                String entryName = fileName.toLowerCase();
                                fileName = fileName.replace(
                                        File.pathSeparatorChar, '_').replace(
                                        File.separatorChar, '_').replace(':',
                                        '|').replace('\'', '_').replace('/',
                                        '_');
//                                logger.debug(entryName);
                                if(PJRef.isPJ(ze)) {
                                    PJRef pj = new PJRef(ze);
                                    pj.writeTmpFile(tmpDir,zf);
                                    ic.getPjs().add(pj);
                                } else if ((entryName.endsWith(dm.getExtension().toLowerCase()) || entryName.endsWith(".xml")) && !fileName.startsWith("_")) {
                                    // on decompresse le fichier dans le
                                    // repertoire temporaire, comme ca il sera
                                    // supprime en quittant
                                    InputStream is = zf.getInputStream(ze);
                                    BufferedInputStream bis = new BufferedInputStream(is);
                                    File output = new File(tmpDir, fileName);
                                    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(output));
                                    byte[] buffer = new byte[1024];
                                    int read = bis.read(buffer);
                                    while (read > 0) {
                                        bos.write(buffer, 0, read);
                                        read = bis.read(buffer);
                                    }
                                    bos.flush();
                                    bos.close();
                                    bis.close();
                                    ic.getFilesToImport().add(output);
                                }
                            }
                        }
                        zf.close();
                    } catch (ZipException zEx) {
                        JOptionPane.showMessageDialog(MainWindow.instance, "Le fichier "
                                + zip.getName()
                                + " n'est pas une archive ZIP valide",
                                "Erreur", JOptionPane.ERROR_MESSAGE);
                    } catch (IOException ioEx) {
                        JOptionPane
                                .showMessageDialog(
                                MainWindow.instance,
                                "Le fichier "
                                + zip.getName()
                                + " est illisible. Vrifiez que les noms des fichiers\ncontenus dans l'archive ne comportent pas de caractres spciaux ou accentus.",
                                "Erreur", JOptionPane.ERROR_MESSAGE);
                        ioEx.printStackTrace();
                    } finally {
                        if(zf!=null) {
                            try { zf.close(); } catch(Throwable t) {}
                        }
                    }

                    fireProperty("terminated", "Gnration termine.");	// histoire de pas faire un new

    				dirEDMN.delete();
//    				for(File f:dirEDMN.listFiles()){
//    					ic.getFilesToImport().add(f);
//    				}
//    				fToImport.setFilesToImport(array);
    				MainWindow.instance.doImport(ei,ic,dm,false);
    			} catch (IOException e) {
    				e.printStackTrace();    				
    			} catch (ClassNotFoundException e) {
    				e.printStackTrace();
    			} catch (InstantiationException e) {
    				e.printStackTrace();
    			} catch (IllegalAccessException e) {
    				e.printStackTrace();
    			} catch (SecurityException e) {
    				e.printStackTrace();
    			} catch (IllegalArgumentException e) {
    				e.printStackTrace();
    			}
    		/*}else if(JOptionPane.NO_OPTION == reponse){
    			DlgEdmnSaveAs dlgsaveas = new DlgEdmnSaveAs(MainWindow.instance,true,dirEDMN);    		
    			dlgsaveas.setVisible(true);
    			int reponse2 = JOptionPane.showConfirmDialog(MainWindow.instance,"Les fichiers gnrs se trouvent dans :\n"+dirEDMN.getAbsolutePath()+"\n\nDsirez-vous ouvrir ce dossier ?","Question",JOptionPane.YES_NO_OPTION);
        		if(JOptionPane.YES_OPTION == reponse2){
        			 try {
						Runtime.getRuntime().exec("explorer.exe \""+dirEDMN.getAbsolutePath()+"\"");
					} catch (IOException e) {
						e.printStackTrace();
					}
        		}*/
    			//JOptionPane.showMessageDialog(MainWindow.instance,"Les fichiers gnrs se trouvent dans :\n"+dirEDMN.getAbsolutePath(),"Information",JOptionPane.INFORMATION_MESSAGE);
    		//}
    	}
        fireProperty("terminated", "");	// histoire de pas faire un new
    }
    
    private File makeZip() throws IOException{
    	 //Cration d'un flux d'criture vers un fichier
    	File fout = new File(dirEDMN.getParent(),"EDMN.xml.zip");
    	FileOutputStream fos=null;
        try {
            fos = new FileOutputStream(fout);
            //Ajout du checksum : Adler32 (plus rapide) ou CRC32
            CheckedOutputStream checksum = new CheckedOutputStream(fos, new Adler32());
            try {
                //Cration d'un buffer de sortie afin d'amliorer les performances
                BufferedOutputStream bos = new BufferedOutputStream(checksum, BUFFER_SIZE);
                try {
                    //Cration d'un flux d'criture Zip vers le fichier  travers le buffer
                    ZipOutputStream zos = new ZipOutputStream(bos);
                    try {
                        //Compression maximale
                        try {
                            zos.setMethod(ZipOutputStream.DEFLATED);
                            zos.setLevel(Deflater.BEST_COMPRESSION);
                        } catch (Exception ignor) {}
                        zipDir(dirEDMN.getAbsolutePath(), null, zos);
                    } finally {
                        zos.close();
                    }
                }finally {
                    bos.close();
                }
            } finally {
                checksum.close();
            }
        } finally {         
			fos.close();            
        }
        return fout;
    }
    private static void zipDir(String basedir, String currentdir, ZipOutputStream zos) throws FileNotFoundException, IOException {
		//create a new File object based on the directory we have to zip
		File zipDir = (currentdir != null) ? new File(basedir, currentdir) : new File(basedir);
		//get a listing of the directory content
		String[] dirList = zipDir.list();
		byte[] readBuffer = new byte[BUFFER_SIZE];
		int bytesIn = 0;
		//On met pas File.separator, mais "/" parce que c'est le caractre utilis
		//dans les ZIP.
		String currentdir2 = (currentdir != null) ? (currentdir + "/") : "";

		File f;
		FileInputStream fis;
		BufferedInputStream bis;
		ZipEntry anEntry;
		//Create an empty entry with just dir name, like WinZip, so unzipToDir will
		//behave correctly.
		if (currentdir2.length() > 0) {
			anEntry = new ZipEntry(currentdir2);
			zos.putNextEntry(anEntry);
			zos.closeEntry();
		}
		//loop through dirList, and zip the files
		for (int i = 0; i < dirList.length; i++) {
			f = new File(zipDir, dirList[i]);
			if (!f.exists())
				continue;
			if (f.isDirectory()) {
				//if the File object is a directory, call this
				//function again to add its content recursively
				zipDir(basedir, currentdir2 + dirList[i], zos);
				continue;
			}
			//if we reached here, the File object f was not a directory
			//create a FileInputStream on top of f
			fis = new FileInputStream(f);
			try {
				bis = new BufferedInputStream(fis, BUFFER_SIZE);
				try {
					//create a new zip entry
					anEntry = new ZipEntry(currentdir2 + dirList[i]);
					anEntry.setTime(f.lastModified());

					//place the zip entry in the ZipOutputStream object
					zos.putNextEntry(anEntry);
					//now write the content of the file to the ZipOutputStream
					while ((bytesIn = bis.read(readBuffer, 0, BUFFER_SIZE)) != -1) {
						zos.write(readBuffer, 0, bytesIn);
					}
					zos.closeEntry();
				}
				finally {
					bis.close();
				}
			}finally {
				//close the Stream
				fis.close();
			}
		}
	}
}
