/*
 * Copyright
 *   2008 axYus - www.axyus.com
 *   2008 L.Meckert - laurent.meckert.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.plugin.signerfichier;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.apache.log4j.Logger;
import java.util.Enumeration;

import fr.gouv.finances.cp.xemelios.ui.MainWindow;
import fr.gouv.finances.cp.xemelios.common.config.DocumentModel;
import fr.gouv.finances.cp.xemelios.common.config.DocumentsModel;
import fr.gouv.finances.cp.xemelios.common.config.TraitementExterneModel;

/**
 *
 * @author laurent.meckert
 */
public class Actions {
   // classe Singleton
   private static Actions _instance = null;
   private File fileToSign = null;
   private File fileToGenerate = null;
   private File certFile = null;
   static Logger logger = Logger.getLogger(Actions.class);
   protected JFileChooser fc;
   IFileSigner signer = null;
   private boolean busy = false;
   private java.awt.Frame parentWindow;
   private DlgKeystore dialogKeystore;

   private Actions() {
      _instance = this;
      initSigner();
   }

   /**
    * 
    * @return la seule instance de la classe 
    */
   public static Actions getInstance() {
      if (_instance == null) {
         _instance = new Actions();
      }
      return _instance;
   }

   public void dispose() {
      if(signer != null) {
         signer.dispose();
         signer = null;
      }
      if(dialogKeystore!= null){
         dialogKeystore.dispose();
         dialogKeystore=null;
      }
          
   }

   /**
    * 
    */
   public void ShowInputFileSelector() {
      if (getFileToSign() != null) {
         fc = new JFileChooser(getFileToSign());
      } else {
         fc = new JFileChooser(".");
      }
      fc.addChoosableFileFilter(new FileFilter() {

            public boolean accept(File f) {
               return f.isDirectory() || f.getName().toUpperCase().endsWith("XML");
            }

            public String getDescription() {
               return "Document XML";
            }
         });
      // Show open dialog; this method does not return until the dialog is closed
      fc.showOpenDialog(null);
      setFileToSign(fc.getSelectedFile());
    
   }

   /**
    * 
    */
   public void ShowCertFileSelector() {
      if (getCertFile() != null) {
         fc = new JFileChooser(getCertFile());
      } else {
         fc = new JFileChooser(".");
      }
      fc.addChoosableFileFilter(new FileFilter() {

            public boolean accept(File f) {
               return f.isDirectory() || f.getName().toUpperCase().endsWith("KS") || f.getName().toUpperCase().endsWith("JKS") || f.getName().toUpperCase().endsWith("JCE") || f.getName().toUpperCase().endsWith("P12") || f.getName().toUpperCase().endsWith("PFX");
            }

            public String getDescription() {
               return "Magasins de cls (.ks, .jks, .jce, .p12, .pks)";
            }
         });
      // Show open dialog; this method does not return until the dialog is closed
      fc.showOpenDialog(null);
      setCertFile(fc.getSelectedFile());
    

   }

   public void ShowOutputFileSelector() {
      // Show save dialog; this method does not return until the dialog is closed
      if (getFileToGenerate() != null) {
         fc = new JFileChooser(getFileToGenerate());
      } else {
         fc = new JFileChooser(".");
      }
      fc.addChoosableFileFilter(new FileFilter() {

            public boolean accept(File f) {
               return f.isDirectory() || f.getName().toUpperCase().endsWith("XML");
            }

            public String getDescription() {
               return "Document XML";
            }
         });
      fc.showSaveDialog(null);
      setFileToGenerate(fc.getSelectedFile());
      if (getFileToGenerate() == null) {
         return;
      }
   }

   //   protected void initSigner() {
   //     FileSignerFactory fsf = FileSignerFactory.newInstance();
   //     fsf.setAttribute(FileSignerFactory.USE_APACHEXMLSEC, true);
   //     //   fsf.setNsCtx(docModel.getNamespaces());

   //     try {
   //       signer = fsf.newFileSigner();
   //     // exmpel  signer.addIntersectFilter("//n:Bordereau");
   //     } catch (Exception e) {
   //       e.printStackTrace();
   //     }
   //   }
   protected void initSigner() {
 
      //later private TraitementExterneModel texModel = null;
      DocumentModel docModel = null;
      File modelBaseDirectory;
      DocumentsModel docModels = null;
      docModels = MainWindow.instance.getDocModels();
      docModel = docModels.getDocumentById("PES_Aller");
      logger.info(docModel);
      modelBaseDirectory = new File(docModel.getBaseDirectory());
      // logger.info(docModel);
      // later texModel = docModel.getTraitementExterneById("signer-fichier-pes");
  
      FileSignerFactory fsf = FileSignerFactory.newInstance();
      fsf.setAttribute(FileSignerFactory.USE_APACHEXMLSEC, true);
      fsf.setNsCtx(docModel.getNamespaces());
    
      try {
         signer = fsf.newFileSigner();
         // exmpel  signer.addIntersectFilter("//n:Bordereau");
      } catch (Exception e) {
         e.printStackTrace();
      }
   }
 

   /**
    * 
    */
   public void chooseInputFile() {
      ShowInputFileSelector();
      if(getFileToSign() == null) return;
      signer.setInputFile(getFileToSign());
      try {
         setBusy(true);
         signer.loadInputFile();
      } catch (Exception e) {
         e.printStackTrace();
      } catch (Error e) {
         e.printStackTrace();
      } finally {
         setBusy(false);
      }
      if (!signer.parseOK()) {
         showStatus();
      } else {
         //jButtonCertFile.setEnabled(true);
      }
   }

   /**
    * 
    */
   public void chooseCertFile() {
      ShowCertFileSelector();

      signer.setKeystoreFile(getCertFile());
      if (dialogKeystore == null) {
         JFrame frame = new JFrame("test");
         dialogKeystore = new DlgKeystore(frame, true);
      }

      dialogKeystore.setSigner(signer);
      dialogKeystore.setTitle("Ouverture du fichier " + getCertFile().getName());
      try {
         dialogKeystore.setMode(DlgKeystore.MODE_KEYSTORE);
      } catch (Exception e) {
         e.printStackTrace();
      }
      dialogKeystore.setVisible(true);
   }

   public void chooseOutputFile() {
      ShowOutputFileSelector();
      if (getFileToGenerate() == null) {
         return;
      }
      if (!fileToGenerate.getName().toUpperCase().endsWith("XML")) {
         setFileToGenerate(new File(getFileToGenerate().getPath() + ".xml"));
      }
      signer.setOutputFile(getFileToGenerate());
   }

   public Enumeration getAliases() throws Exception {
      if (signer != null && signer.loadKsOK()) {
         return signer.getAliases();
      } else {
         return null;
      }
   }

   public boolean loadAlias(Object o) {

      if (signer != null) {
         try {
            signer.setAlias((String) (o));
            signer.setAliasPassword(signer.getKeystorePassword());
            // Premier essai avec le mot de passe generique du fichier 
            signer.loadAlias();

            // Si a ne marche pas on demande le mdp specifique de la cle
            if (!signer.loadPkOK()) {
               dialogKeystore.setTitle("Ouverture de la cle prive  " + signer.getAlias());
               dialogKeystore.setMode(DlgKeystore.MODE_ALIAS);
               dialogKeystore.setVisible(true);
            }
            return signer.loadPkOK();


         } catch (Exception e) {
            return false;
         }
      } else {
         return false;
      }
   }
  
   public String getAliasName(){
      return signer.getAlias();
   }

   public File getFileToSign() {
      return fileToSign;
   }

   public void setFileToSign(File fileToSign) {
      this.fileToSign = fileToSign;
   }
  
   public String getSignStatusString() {
      String bla;
      if (signer != null) {
         bla = signer.getStatusAsString();
      } else {
         bla = "Une erreur inattendue est survenue; veuillez consulter le log de l'application";
      }
      return bla;
   }

   /**
    * Montre le rsultat de la vrification
    * @param signer 
    */
   public void showStatus() {
      String bla;
      int sever;
      if (signer != null) {
         bla = signer.getStatusAsString();
         sever = signer.getStatusAsSeverity();
      } else {
         sever = FileSigner.SIGN_VERYBAD;
         bla = "Une erreur inattendue est survenue; veuillez consulter le log de l'application";
      }

      JOptionPane.showMessageDialog(getParentWindow(), bla,
                                    "Rsultat de la vrification",
                                    sever == FileSigner.SIGN_VERYBAD ? JOptionPane.ERROR_MESSAGE : (sever == FileSigner.SIGN_BAD ? JOptionPane.WARNING_MESSAGE : JOptionPane.PLAIN_MESSAGE));
   }

   public boolean isBusy() {
      return this.busy;
   }

   void setBusy(boolean busy) {
      // only set if changing
      if (this.busy != busy) {
         this.busy = busy;
      }
   }

   public java.awt.Frame getParentWindow() {
      return parentWindow;
   }

   public void setParentWindow(java.awt.Frame parentWindow) {
      this.parentWindow = parentWindow;
   }

   public File getCertFile() {
      return certFile;
   }

   public void setCertFile(File certFile) {
      this.certFile = certFile;
   }

   public File getFileToGenerate() {
      return fileToGenerate;
   }

   public void setFileToGenerate(File fileToGenerate) {
      this.fileToGenerate = fileToGenerate;
   }
   public void doSign() throws Exception{
      signer.sign();    
   }
}
