/*
 * Copyright 
 *   2005 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.auth;

import java.util.Properties;

/**
 * Defines an authentication system. An <tt>AuthenticationManager</tt> is responsible
 * for initializing the authentication persistence layer, furnishing {@link fr.gouv.finances.cp.xemelios.auth.AuthProvider}
 * and {@link fr.gouv.finances.cp.xemelios.auth.AuthChecker}
 * @author chm
 *
 */
public interface AuthenticationManager {
	/**
	 * Returns the checker used by this manager
	 * @return
	 */
	public AuthChecker getChecker();
	/**
	 * Returns the provider used by this manager
	 * @return
	 */
	public AuthProvider getProvider();
	/**
	 * Returns this AuthenticationManager's name
	 * @return
	 */
	public String getAuthenticationManagerName();
	/**
	 * This method is called by AuthenticationFactory to initialize manager with configuration file
	 * @param props
	 */
	public void setProperties(Properties props);
	/**
	 * This method is called by AuthenticationFactory to allow manager interact with mainWindow
	 * @param mw
	 */
}
