/*
 * Copyright 
 *   2007 axYus - www.axyus.com
 *   2007 C.Marchand - christophe.marchand@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.common.config;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.NoeudModifiable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import fr.gouv.finances.cp.xemelios.data.DataConfigurationException;
import javax.xml.namespace.QName;

public class ReferenceModel implements NoeudModifiable {
    private static Logger logger = Logger.getLogger(ReferenceModel.class);
    private NoeudModifiable _NMParent = null;
    
    public static final transient String TAG = "reference";
    public static final transient QName QN = new QName(TAG);
    private String name;
    
    public ReferenceModel(QName tagName) {
        super();
    }
    
    public void addCharacterData(String cData) throws SAXException {}
    
    public void addChild(XmlMarshallable child, QName tagName) throws SAXException {}
    
    public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
        name = (attributes.getValue("name")!=null?attributes.getValue("name"):name);
        return this;
    }
    
    public void marshall(XmlOutputter output) {}
    
    public void validate() throws InvalidXmlDefinition {
        if(name==null || name.length()==0) throw new InvalidXmlDefinition("//"+TAG+"/@name can not be null or empty");
    }
    
    public ReferenceModel clone() {
        ReferenceModel other = new ReferenceModel(QN);
        other.name=name;
        return other;
    }
    public String getName() { return name; }
    
    public Object getValue(EnvironmentDomain domain) throws DataConfigurationException {
        String path = getName();
        if(path.startsWith("/")) path = path.substring(1);
        return domain.getValue(path);
    }
    
    public void setValue(EnvironmentDomain domain, Object value) throws DataConfigurationException {
        String path = getName();
        if(path.startsWith("/")) path = path.substring(1);
        domain.setValue(path,value);
    }
    
    public void setParentAsNoeudModifiable(NoeudModifiable p) {
        this._NMParent = p;
    }
    public NoeudModifiable getParentAsNoeudModifiable() {
        return this._NMParent;
    }
    public NoeudModifiable getChildAsNoeudModifiable(String tagName, String id) {
        return null;
    }
    public void modifyAttr(String attrName, String value) {
        // TODO Raccord de mthode auto-gnr
        
    }
    public void modifyAttrs(Attributes attrs) {
        try {
            getAttributes(new XmlAttributes(attrs));
        } catch (Exception e) {
            logger.error("Erreur lors de la mise  jour des attributs : "+e);
        }
    }
    
    public String[] getChildIdAttrName(String childTagName) {
        return null;
    }
    
    public void resetCharData() {
    }
    public String getIdValue() { return null; }
    private String configXPath = null;
    public String getConfigXPath() {
        if(configXPath==null) {
            if(getParentAsNoeudModifiable()!=null) configXPath = getParentAsNoeudModifiable().getConfigXPath();
            else configXPath="";
            configXPath+="/"+TAG+"[@name='"+getName()+"']";
        }
        return configXPath;
    }

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }
}

