package fr.gouv.finances.dgfip.xemelios.services;

import fr.gouv.finances.dgfip.xemelios.data.impl.MySqlDataLayer;
import fr.gouv.finances.dgfip.xemelios.common.config.DocumentModel;
import fr.gouv.finances.dgfip.xemelios.common.config.DocumentsModel;
import fr.gouv.finances.dgfip.xemelios.common.config.ElementModel;
import fr.gouv.finances.dgfip.xemelios.common.config.EtatModel;
import fr.gouv.finances.dgfip.xemelios.common.config.Loader;
import fr.gouv.finances.dgfip.xemelios.data.DataConfigurationException;
import fr.gouv.finances.dgfip.xemelios.data.DataLayerManager;
import fr.gouv.finances.dgfip.xemelios.web.utils.ParametreManager;
import java.io.IOException;
import java.util.Enumeration;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/**
 * @author CBO
 */
public class XemeliosServiceFacade {

    static public DocumentsModel documentsModel;

    static public void init() {
        
        try {
            documentsModel = Loader.getDocumentsInfos(ParametreManager.getDocumentdir());
        } catch (SAXException ex) {
            throw new RuntimeXemeliosServiceException("Erreur lors du chargement des documents depuis " + ParametreManager.getDocumentdir(), ex);
        } catch (ParserConfigurationException ex) {
            throw new RuntimeXemeliosServiceException("Erreur lors du chargement des documents depuis " + ParametreManager.getDocumentdir(), ex);
        } catch (IOException ex) {
            throw new RuntimeXemeliosServiceException("Erreur lors du chargement des documents depuis " + ParametreManager.getDocumentdir(), ex);
        }
        try {
            MySqlDataLayer mySqlDataLayer = new MySqlDataLayer();
            DataLayerManager.regiterDataImpl(mySqlDataLayer.getClass().getName(), mySqlDataLayer.getClass());
            DataLayerManager.setDataImpl(MySqlDataLayer.class.getName());
        } catch(DataConfigurationException dce) {
            throw new RuntimeXemeliosServiceException("Erreur lors de la configuration de la couche d'accs aux donnes", dce);
        }
    }

    static public DocumentModel getDocumentsModel(String documentId) {
        for (DocumentModel dm : documentsModel.getDocuments()) {
            if (documentId.equals(dm.getId())) {
                return dm;
            }
        }
        return null;
    }
    
    static public Menu loadMenu() throws RuntimeXemeliosServiceException {
        Menu rootMenu = new Menu(null, null);
        Menu searchMenu = new Menu(null, "Recherche");
        rootMenu.addChild(searchMenu);
        for (DocumentModel dm : documentsModel.getDocuments()) {
            Enumeration<EtatModel> enumer2 = dm.getEtats().elements();
            Menu searchEntry = new Menu(null, dm.getTitre());
            int searchCount = 0;
            while (enumer2.hasMoreElements()) {
                EtatModel em = enumer2.nextElement();
                if (em.getDisplayInMenuIf() != null && !"true".equals(System.getProperty(em.getDisplayInMenuIf()))) {
                    continue;
                }
                Menu tmpMnu = null;
                if (!dm.isGroupMenuItem()) {
                    tmpMnu = new Menu(null, em.getTitre());
                } else {
                    tmpMnu = searchEntry;
                }
                for (ElementModel elm : em.getSearchableElements()) {
                    if (!elm.isDisplay()) {
                        continue;
                    }
                    tmpMnu.addChild(new Menu("search.do?documentId=" + dm.getId() + "&etatId=" + em.getId() + "&elementId=" + elm.getId(), elm.getTitre()));
                    searchCount++;
                }
                if (tmpMnu != searchEntry) {
                    if (tmpMnu.getChilds().size() > 0) {
                        searchEntry.addChild(tmpMnu);
                    }
                }
            }
            if (searchCount > 0) {
                searchMenu.addChild(searchEntry);
            }
        }
        return rootMenu;
    }
}
