package fr.gouv.finances.dgfip.xemelios.web;

import fr.gouv.finances.dgfip.xemelios.common.Constants;
import fr.gouv.finances.dgfip.xemelios.data.impl.MySqlDataLayer;
import fr.gouv.finances.dgfip.xemelios.services.Menu;
import fr.gouv.finances.dgfip.xemelios.services.XemeliosServiceFacade;
import fr.gouv.finances.dgfip.xemelios.web.utils.ParametreManager;
import java.io.File;
import javax.servlet.ServletContextEvent;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * @author CBO
 */
public class Application {

    private static Logger logger = org.apache.log4j.Logger.getLogger(Application.class);
    static private final String configFileParameterName = "lo4j";
    static private Menu rootMenu = null;

    static public void start(ServletContextEvent event) {
        try {
            initLog4j(event);
        } catch (RuntimeException re) {
            re.printStackTrace();
            throw (re);
        }
        try {
            System.setProperty(Constants.SYS_PROP_DOC_DEF_DIR, ParametreManager.getDocumentdir());
            System.setProperty(MySqlDataLayer.PROP_CONFIG_URL, event.getServletContext().getResource("/WEB-INF/mysql.properties").toExternalForm());
            XemeliosServiceFacade.init();
            rootMenu = XemeliosServiceFacade.loadMenu();
        } catch (Exception e) {
            logger.error(e, e);
            throw new RuntimeException(e);
        }
    }

    static public void stop(ServletContextEvent event) {
        LogManager.shutdown();
    }

    static public Menu getMenu() {
        return rootMenu;
    }

    static private void initLog4j(ServletContextEvent event) {
        String confFileAsString = event.getServletContext().getInitParameter(configFileParameterName);
        if (confFileAsString == null) {
            confFileAsString = "./WEB-INF/log4j.xml";
        }
        File confFile = null;
        if (confFileAsString.startsWith("./")) {
            confFile = new File(event.getServletContext().getRealPath("/"), confFileAsString.substring(2));
        } else {
            confFile = new File(confFileAsString);
        }
        event.getServletContext().log("Initialisation de Log4J  partir du fichier : " + confFile);
        if (!confFile.exists()) {
            throw new IllegalArgumentException("Fichier de configuration log4j spcifi non trouvable : " + confFile.getAbsolutePath());
        }
        System.setProperty("webApp.home", event.getServletContext().getRealPath("/"));
        System.setProperty("webApp.name", new File(event.getServletContext().getRealPath("/")).getName());
        DOMConfigurator.configureAndWatch(confFile.getAbsolutePath());
    }
}
