package fr.gouv.finances.dgfip.xemelios.web;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.RequestProcessor;

/**
 * @author CBO
 * Ce RequestProcessor permet de :
 * <li>Grer l'expiration de session</li>
 * <li>Grer les droits en terme de fonctionnalits de l'utilisateur</li>
 */
public class CustomRequestProcessor extends RequestProcessor {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(CustomRequestProcessor.class);

    @Override
    protected boolean processRoles(HttpServletRequest request, HttpServletResponse response, ActionMapping mapping) throws IOException, ServletException {
        String roles[] = mapping.getRoleNames();
        if ((roles == null) || (roles.length < 1)) {
            return true;
        }
        Principal principal = AttributeHelper.getPrincipal(request.getSession());
        if (principal == null) {
            ActionMessages actionMessages = new ActionMessages();
            actionMessages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("connexion.sessionExpiree"));
            request.getSession().setAttribute(Globals.ERROR_KEY, actionMessages);

            if ("XMLHttpRequest".equals(request.getHeader("X-Requested-With"))) {
                response.sendError(response.SC_FORBIDDEN, request.getContextPath());
            } else {
                response.sendRedirect(request.getContextPath());
            }
            return false;
        } else {
            for (int i = 0; i < roles.length; i++) {
                if (principal.getCodesFonctionnalites() != null && principal.getCodesFonctionnalites().contains(roles[i])) {
                    return true;
                }
            }
            logger.warn("Tentative d'accs  une fonctionnalit non autorise, une des fonctionnalits suivantes taient requises : " + getRolesAsString(roles));
            response.sendRedirect(request.getContextPath() + "/nonAutorise.jsp");
            //response.sendError(HttpServletResponse.SC_FORBIDDEN, getInternal().getMessage("notAuthorized", mapping.getPath()));
            return false;
        }
    }

    private String getRolesAsString(String roles[]) {
        StringBuilder sb = new StringBuilder();
        boolean firstLoop = true;
        for (String role : roles) {
            if (!firstLoop) {
                sb.append(", ");
            }
            sb.append(role);
            firstLoop = false;
        }
        return sb.toString();
    }
}
