package fr.gouv.finances.dgfip.xemelios.web;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * @author CBO
 * Cette interface est utilise pour dterminer les droits et le nom d'affichage de l'utilisateur connect.
 */
public interface Principal {

  public String getDisplayName();

  public Collection<String> getCodesFonctionnalites();

  public boolean isAdmin();

  public List<CodicCodeCol> getCodiqueCodeCols();

  public List<String> getCodiqueCodeColCodes();

  static public class CodicCodeCol implements Comparable<CodicCodeCol> {

    private String codic;
    private String codeCol;

    static public List<CodicCodeCol> fromCodes(String codiqueCodeColCodes[]) {
      List<CodicCodeCol> toReturn = new ArrayList<CodicCodeCol>();
      if (codiqueCodeColCodes != null) {
        for (String codiqueCodeColCode : codiqueCodeColCodes) {
          if (codiqueCodeColCode != null) {
            toReturn.add(new CodicCodeCol(codiqueCodeColCode));
          }
        }
      }
      return toReturn;
    }

    public CodicCodeCol(String codiqueCodeCol) {
      this.codic = codiqueCodeCol.substring(0, 6);
      this.codeCol = codiqueCodeCol.substring(6, 9);
    }

    public String getCodeCol() {
      return codeCol;
    }

    public String getCodic() {
      return codic;
    }

    public String getCode() {
      return getCodic() + getCodeCol();
    }

    public String getLibelle() {
      return getCodic() + getCodeCol();
    }

    @Override
    public boolean equals(Object obj) {
      if (obj == null) {
        return false;
      }
      if (getClass() != obj.getClass()) {
        return false;
      }
      final CodicCodeCol other = (CodicCodeCol) obj;
      if (this.codic != other.codic && (this.codic == null || !this.codic.equals(other.codic))) {
        return false;
      }
      if (this.codeCol != other.codeCol && (this.codeCol == null || !this.codeCol.equals(other.codeCol))) {
        return false;
      }
      return true;
    }

    @Override
    public int hashCode() {
      int hash = 3;
      hash = 47 * hash + (this.codic != null ? this.codic.hashCode() : 0);
      hash = 47 * hash + (this.codeCol != null ? this.codeCol.hashCode() : 0);
      return hash;
    }

    public int compareTo(CodicCodeCol other) {
      return getCode().compareTo(other.getCode());
    }
  }
}
