package fr.gouv.finances.dgfip.xemelios.web.connexion;

import fr.gouv.finances.dgfip.xemelios.web.utils.ParametreManager;
import fr.gouv.finances.dgfip.xemelios.web.Principal;
import fr.gouv.finances.dgfip.xemelios.web.Principal.CodicCodeCol;
import fr.gouv.finances.dgfip.xemelios.web.connexion.LDAPHabilitationParser.LDAPHabilitations;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Date;
import javax.naming.directory.DirContext;
import javax.servlet.http.HttpServletRequest;
import org.apache.log4j.Logger;

/**
 * Le webSSO (lemonLDAP) transmet un header contenant le dn suivit d'un profil
 * Exemple : uid=test-cp,ou=personnes,ou=personnes,ou=dgcp,ou=mefi,o=gouv,c=fr:compt
 * Ensuite grace au dn on r�cup�re des donn�es d'habilitation au format xml dans l'annuaire LDAP
 * @author CBO
 */
public class LDAPAuthenticatorImpl implements Authenticator {

  private static Logger logger = org.apache.log4j.Logger.getLogger(LDAPAuthenticatorImpl.class);

  public Principal authenticate(HttpServletRequest request) throws AuthenticatorException, SQLException {
    String requiredProfilName = ParametreManager.getProfilApplicatifRequis();
    String authHeaderName = ParametreManager.getAuthHeaderName();
    String authHeader = request.getHeader(authHeaderName);
    logger.debug("Lecture du header d'authentification (" + authHeaderName + "): " + authHeader);
    if (authHeader == null) {
      throw new AuthenticatorException("Header d'authentification (" + authHeaderName + ") introuvable");
    }
    if (authHeader.lastIndexOf(":") < 0) {
      throw new AuthenticatorException("Header d'authentification n'est pas au format attendu : " + authHeader);
    }
    String dn = authHeader.substring(0, authHeader.lastIndexOf(":")).trim();
    String profil = authHeader.substring(authHeader.lastIndexOf(":") + 1).trim();
    if (!requiredProfilName.equalsIgnoreCase(profil)) {
      logger.info("Authentification refusée, le profil de l'utilisateur est " + profil + ", le profil requis est " + requiredProfilName);
      return null;
    } else {
      return getPrincipalFromLDAP(dn);
    }
  }

  private Principal getPrincipalFromLDAP(String dn) throws SQLException {
    DirContext dirContext = null;
    try {
      String urlLDAP = ParametreManager.getUrlLdap();
      String urlLDAPSecour = ParametreManager.getUrlLdapSecours();
      String userApplicatif = ParametreManager.getUserApplicatifLdap();
      String passwordApplicatif = ParametreManager.getPasswordApplicatifLdap();
      dirContext = LDAPUtils.createLDAPContext(urlLDAP, urlLDAPSecour, userApplicatif, passwordApplicatif);
      LDAPUser ldapUser = (LDAPUser) LDAPUtils.getObjectByDN(dirContext, LDAPUser.class, dn);
      if (ldapUser == null) {
        return null;
      } else {
        Date now = new Date();
        String habilitationsHelios = ldapUser.getHabilitation(ParametreManager.getCodeApplicationLDAP());
        LDAPHabilitations habilitations = LDAPHabilitationParser.parse(habilitationsHelios);
        boolean isAdmin = habilitations.isAllowedToCodique(now, ParametreManager.getCodiqueAdministrateur());
        return new PrincipalImpl(isAdmin, ldapUser.getDisplayName(), Collections.<String>emptyList(), Collections.<CodicCodeCol>emptyList());
      }
    } finally {
      LDAPUtils.closeLDAPContext(dirContext);
    }
  }

  static public class LDAPUser {

    private String cn;
    private String givenName;
    private String[] mefiAppliHabilitDGCP;

    public String getCn() {
      return cn;
    }

    public void setCn(String cn) {
      this.cn = cn;
    }

    public String getGivenName() {
      return givenName;
    }

    public void setGivenName(String givenName) {
      this.givenName = givenName;
    }

    public String[] getMefiAppliHabilitDGCP() {
      return mefiAppliHabilitDGCP;
    }

    public void setMefiAppliHabilitDGCP(String[] mefiAppliHabilitDGCP) {
      this.mefiAppliHabilitDGCP = mefiAppliHabilitDGCP;
    }

    public String getDisplayName() {
      return getGivenName() + " " + getCn();
    }

    public String getHabilitation(String codeApplication) {
      if (getMefiAppliHabilitDGCP() != null) {
        for (String habilitation : getMefiAppliHabilitDGCP()) {
          habilitation = habilitation != null ? habilitation.trim() : null;
          if (habilitation != null && habilitation.toUpperCase().startsWith(codeApplication.toUpperCase())) {
            int sepPos = habilitation.indexOf(";");
            if (sepPos >= 0) {
              habilitation = habilitation.substring(sepPos + 1);
              sepPos = habilitation.indexOf(";");
              if (sepPos >= 0) {
                habilitation = habilitation.substring(0, sepPos);
              }
            }
            return habilitation;
          }
        }
      }
      return null;
    }
  }
}
