package fr.gouv.finances.dgfip.xemelios.web.connexion;

import fr.gouv.finances.dgfip.xemelios.web.utils.DateUtils;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author CBO
 */
public class LDAPHabilitationParser {

    private static Logger logger = org.apache.log4j.Logger.getLogger(LDAPHabilitationParser.class);
    private static final String HABILITATION_NN = "HABILITATION";
    private static final String PERIODE_NN = "PERIODE";
    private static final String START_NN = "START";
    private static final String END_NN = "END";
    private static final String ACL_NN = "ACL";
    private static final String CODE_STRUCTURE_SUP_NN = "CODE_STRUCTURE_SUP";
    private static final String CODE_BUDCOLL_NN = "CODE_BUDCOLL";
    private static final String ACLTYPE_NN = "ACLTYPE";
    private static final String ACLTYPE_ALLOW_NV = "ALLOW";
    private static final String ACL_VALUE_NN = "ACLVALUE";
    private static final String BUDCOL_VALUE_PN = "BUDCOL_VALUE";
    private static final String CODIQUE_VALUE_PN = "CODIQUE_VALUE";

    static public LDAPHabilitations parse(String xml) {
        try {
            LDAPHabilitations toReturn = new LDAPHabilitations();
            if (xml != null && xml.trim().length() > 0) {
                Document document = null;
                byte[] xmlByteArray = xml.getBytes();
                InputStream xmlStream = new ByteArrayInputStream(xmlByteArray);
                document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(xmlStream));
                NodeList habilitationNodes = document.getElementsByTagName(HABILITATION_NN);
                for (int i = 0; habilitationNodes != null && i < habilitationNodes.getLength(); i++) {
                    Node habilitationNode = habilitationNodes.item(i);
                    LDAPHabilitation habilitation = new LDAPHabilitation();
                    habilitation.setDateDebut(getChildNodeValueAsDate(habilitationNode, new String[]{PERIODE_NN, START_NN}));
                    habilitation.setDateFin(getChildNodeValueAsDate(habilitationNode, new String[]{PERIODE_NN, END_NN}));
                    // Codiques ( codique = poste comptable )
                    Node codeStructureSupNode = getChildNode(habilitationNode, new String[]{ACL_NN, CODE_STRUCTURE_SUP_NN});
                    if (codeStructureSupNode != null) {
                        String codeStructureSupAclType = getChildNodeValueAsString(codeStructureSupNode, new String[]{ACLTYPE_NN});
                        if (ACLTYPE_ALLOW_NV.equalsIgnoreCase(codeStructureSupAclType)) {
                            List<Node> aclValueNodes = getChildNodes(codeStructureSupNode, ACL_VALUE_NN);
                            for (Node aclValueNode : aclValueNodes) {
                                String value = getChildNodeValueAsString(aclValueNode, null);
                                if (value != null) {
                                    habilitation.addCodique(value);
                                }
                            }
                        }
                    }
                    // IdsOrganime :
                    Node codeBudCollNode = getChildNode(habilitationNode, new String[]{ACL_NN, CODE_BUDCOLL_NN});
                    if (codeBudCollNode != null) {
                        String codeBudCollNodeAclType = getChildNodeValueAsString(codeBudCollNode, new String[]{ACLTYPE_NN});
                        if (ACLTYPE_ALLOW_NV.equalsIgnoreCase(codeBudCollNodeAclType)) {
                            List<Node> aclValueNodes = getChildNodes(codeBudCollNode, ACL_VALUE_NN);
                            for (Node aclValueNode : aclValueNodes) {
                                String budColValue = getChildNodeValueAsString(aclValueNode, new String[]{BUDCOL_VALUE_PN});
                                String codiqueValue = getChildNodeValueAsString(aclValueNode, new String[]{CODIQUE_VALUE_PN});
                                if (budColValue != null && codiqueValue != null) {
                                    habilitation.addOrganismeId(codiqueValue + budColValue);
                                }
                            }
                        }
                    }
                    toReturn.add(habilitation);
                }
            }
            return toReturn;
        } catch (ParserConfigurationException pce) {
            throw new RuntimeException(pce);
        } catch (SAXException ex) {
            throw new RuntimeException(ex);
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    static private String getChildNodeValueAsString(Node parentNode, String[] path) {
        Node node = getChildNode(parentNode, path);
        if (node != null) {
            NodeList nodeList = node.getChildNodes();
            for (int i = 0; nodeList != null && i < nodeList.getLength(); i++) {
                Node textNode = nodeList.item(i);
                if (textNode.getNodeType() == Node.TEXT_NODE) {
                    return textNode.getNodeValue();
                }
            }
        }
        return null;
    }

    static private Date getChildNodeValueAsDate(Node parentNode, String[] path) {
        try {
            String dateAsString = getChildNodeValueAsString(parentNode, path);
            return dateAsString != null ? DateUtils.parseDate(dateAsString) : null;
        } catch (ParseException ex) {
            throw new RuntimeException(ex);
        }
    }

    static private Node getChildNode(Node parentNode, String[] path) {
        Node currentNode = parentNode;
        if (path == null || path.length == 0) {
            return parentNode;
        } else {
            for (int i = 0; i < path.length; i++) {
                currentNode = getChildNode(currentNode, path[i]);
                if (i == path.length - 1) {
                    return currentNode;
                }
            }
            return null;
        }
    }

    static private Node getChildNode(Node parentNode, String name) {
        if (parentNode == null) {
            return null;
        } else {
            NodeList nodeList = parentNode.getChildNodes();
            for (int i = 0; i < nodeList.getLength(); i++) {
                Node node = nodeList.item(i);
                if (name.equalsIgnoreCase(node.getNodeName())) {
                    return node;
                }
            }
            return null;
        }
    }

    static private List<Node> getChildNodes(Node parentNode, String name) {
        List<Node> toReturn = new ArrayList<Node>();
        NodeList nodeList = parentNode.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (name.equalsIgnoreCase(node.getNodeName())) {
                toReturn.add(node);
            }
        }
        return toReturn;
    }

    static public class LDAPHabilitations {

        private List<LDAPHabilitation> habilitations = new ArrayList<LDAPHabilitation>();

        public LDAPHabilitations() {
        }

        public void add(LDAPHabilitation habilitation) {
            this.habilitations.add(habilitation);
        }

        public List<LDAPHabilitation> getAll() {
            return habilitations;
        }

        public List<LDAPHabilitation> getAllActive(Date date) {
            List<LDAPHabilitation> toReturn = new ArrayList<LDAPHabilitation>();
            for (LDAPHabilitation habilitation : getAll()) {
                if (habilitation.isActive(date)) {
                    toReturn.add(habilitation);
                }
            }
            return toReturn;
        }

        public List<String> getCodicColCodeCodesActifs(Date date) {
            SortedSet<String> set = new TreeSet<String>();
            for (LDAPHabilitation habilitationsActives : getAllActive(date)) {
                for (String idOrganisme : habilitationsActives.getOrganismeIds()) {
                    if (idOrganisme.length() < 9) {
                        logger.warn("idOrganisme ignor� : " + idOrganisme);
                    } else {
                        set.add(idOrganisme.substring(0, 9));
                    }
                }
            }
            return new ArrayList<String>(set);
        }

        public boolean isAllowedToCodique(Date date, String searchedCodique) {
            for (LDAPHabilitation habilitationsActives : getAllActive(date)) {
                for (String codique : habilitationsActives.getCodiques()) {
                    if (searchedCodique.equalsIgnoreCase(codique)) {
                        return true;
                    }
                }
            }
            return false;
        }
    }

    static public class LDAPHabilitation {

        private List<String> codiques = new ArrayList<String>();
        private List<String> organismeIds = new ArrayList<String>();
        private Date dateDebut;
        private Date dateFin;

        public Date getDateDebut() {
            return dateDebut;
        }

        public void setDateDebut(Date dateDebut) {
            this.dateDebut = dateDebut;
        }

        public Date getDateFin() {
            return dateFin;
        }

        public void setDateFin(Date dateFin) {
            this.dateFin = dateFin;
        }

        public boolean isActive(Date date) {
            return (getDateDebut() != null && !getDateDebut().after(date)) && (getDateFin() == null || !getDateFin().before(date));
        }

        public void addCodique(String codique) {
            this.codiques.add(codique);
        }

        public void addOrganismeId(String id) {
            this.organismeIds.add(id);
        }

        public List<String> getCodiques() {
            return codiques;
        }

        public List<String> getOrganismeIds() {
            return organismeIds;
        }
    }
}
