package fr.gouv.finances.dgfip.xemelios.web.connexion;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NameNotFoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

/**
 * @author CBO
 */
public class LDAPUtils {

  static private org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(LDAPUtils.class);
  static private String INITIAL_CONTEXT_FACTORY = "com.sun.jndi.ldap.LdapCtxFactory";

  static DirContext createLDAPContext(String url, String urlSecours) {
    return createLDAPContext(url, urlSecours, null, null);
  }

  public static DirContext createLDAPContext(String url, String urlSecours, String user, String password) {
    try {
      return createLDAPContext(url, user, password);
    } catch (NamingException ex1) {
      if (urlSecours != null) {
        try {
          // On retente sur l'url de secour
          return createLDAPContext(urlSecours, user, password);
        } catch (NamingException ex2) {
          throw new RuntimeException(ex2);
        }
      } else {
        throw new RuntimeException(ex1);
      }
    }
  }

  public static void closeLDAPContext(DirContext context) {
    try {
      context.close();
    } catch (NamingException ex) {
      throw new RuntimeException(ex);
    }
  }

  static public Object getObjectByDN(DirContext context, Class clazz, String dn) {
    Field[] fields = clazz.getDeclaredFields();
    String[] attributsALire = new String[fields.length];
    for (int i = 0; i < fields.length; i++) {
      final Field field = fields[i];
      checkModifier(field);
      attributsALire[i] = field.getName();
    }
    Object toReturn = createInstance(clazz);
    try {
      Attributes attributes = context.getAttributes(dn, attributsALire);
      bindAttributes(toReturn, attributes);
      return toReturn;
    } catch (NameNotFoundException nnfe) {
      return null; // Cas ou on a pas trouv�
    } catch (NamingException ex) {
      throw new RuntimeException(ex);
    }
  }

  static private Object createInstance(Class clazz) {
    try {
      return clazz.newInstance();
    } catch (InstantiationException ex) {
      throw new RuntimeException(ex);
    } catch (IllegalAccessException ex) {
      throw new RuntimeException(ex);
    }
  }

  private static void bindAttributes(Object object, Attributes attributes) {
    try {
      NamingEnumeration attributeEnumeration = attributes.getAll();
      while (attributeEnumeration.hasMore()) {
        Attribute attribute = (Attribute) attributeEnumeration.next();
        Field fields[] = object.getClass().getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
          if (fields[i].getName().equalsIgnoreCase(attribute.getID())) {
            bindAttribute(object, fields[i], attribute);
          }
        }
      }
    } catch (NamingException ex) {
      throw new RuntimeException(ex);
    }
  }

  private static void checkModifier(final Field field) {
    if (!Modifier.isPublic(field.getModifiers())) {
      AccessController.doPrivileged(
              new PrivilegedAction() {

                public Object run() {
                  field.setAccessible(true);
                  return null;
                }
              });
    }
  }

  private static void bindAttribute(Object object, Field field, Attribute attribute) {
    try {
      if (field.getType().isArray()) {
        NamingEnumeration enumeration = attribute.getAll();
        ArrayList list = new ArrayList();
        while (enumeration.hasMore()) {
          list.add(enumeration.next());
        }
        checkModifier(field);
        field.set(object, list.toArray((Object[]) Array.newInstance(field.getType().getComponentType(), list.size())));
      } else {
        checkModifier(field);
        field.set(object, attribute.get(0));
      }
    } catch (IllegalArgumentException ex) {
      throw new RuntimeException(ex);
    } catch (IllegalAccessException ex) {
      throw new RuntimeException(ex);
    } catch (NamingException ex) {
      throw new RuntimeException(ex);
    }
  }

  private static DirContext createLDAPContext(String url, String user, String password) throws NamingException {
    logger.debug("Tentative de cr�ation d'une connexion LDAP : " + url);
    Hashtable env = new Hashtable();
    logger.debug("Url du serveur LDAP : " + url);
    env.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
    env.put(Context.PROVIDER_URL, url);
    if (user != null) {
      logger.debug("ConnectLDAP : SECURITY_AUTHENTICATION = simple");
      env.put(Context.SECURITY_AUTHENTICATION, "simple");
      env.put(Context.SECURITY_PRINCIPAL, user);
      if (password != null) {
        env.put(Context.SECURITY_CREDENTIALS, password);
      }
    } else {
      logger.debug("ConnectLDAP : SECURITY_AUTHENTICATION = none (Anonyme)");
      env.put(Context.SECURITY_AUTHENTICATION, "none");
    }

    return new InitialDirContext(env);
  }
}
