package fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ChangeListenerCollection;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.LabelValueDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.ServiceLocator;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.GWTUtils;
import java.util.List;

/**
 * @author CBO
 * Widget permettant  l'utilisateur de choisir la collectivit et le budget
 */
public class ColAndBudChooserWidget extends Composite {

    private SearchWidget searchWidget;
    private ListBox collectivitesListBox;
    private ListBox budgetListBox;
    private ChangeListenerCollection changeListeners;
    private HorizontalPanel horizontalPanel;

    public ColAndBudChooserWidget(final SearchWidget searchWidget) {
        this.searchWidget = searchWidget;
        this.collectivitesListBox = new ListBox();
        this.budgetListBox = new ListBox();
        this.changeListeners = new ChangeListenerCollection();
        collectivitesListBox.addChangeListener(new ChangeListener() {

            public void onChange(Widget widget) {
                collectiviteChange();
            }
        });
        budgetListBox.addChangeListener(new ChangeListener() {

            public void onChange(Widget widget) {
                budgetChange();
            }
        });
        this.horizontalPanel = new HorizontalPanel();
        this.horizontalPanel.add(new Label(new HTML("Collectivit&eacute;").getHTML()));
        this.horizontalPanel.add(this.collectivitesListBox);
        this.horizontalPanel.add(new Label("Code budget"));
        this.horizontalPanel.add(this.budgetListBox);
        initWidget(this.horizontalPanel);
    }

    public void init(List<LabelValueDTO> collectivites, List<LabelValueDTO> budgets) {
        this.collectivitesListBox.clear();
        for (LabelValueDTO collectivite : collectivites) {
            collectivitesListBox.addItem(collectivite.getLabel(), collectivite.getValue());
        }
        for (LabelValueDTO budget : budgets) {
            budgetListBox.addItem(budget.getLabel(), budget.getValue());
        }
    }

    public String getCollectiviteId() {
        return collectivitesListBox.getValue(collectivitesListBox.getSelectedIndex());
    }

    public String getBudgetId() {
        return budgetListBox.getValue(budgetListBox.getSelectedIndex());
    }

    public void addChangeListener(ChangeListener changeListener) {
        this.changeListeners.add(changeListener);
    }

    private void collectiviteChange() {
        ServiceLocator.getXemeliosService().getBudgets(searchWidget.getDocumentId(), getCollectiviteId(), new AsyncCallback<List<LabelValueDTO>>() {

            public void onFailure(Throwable throwable) {
                GWTUtils.handleRPCError(throwable);
            }

            public void onSuccess(List<LabelValueDTO> budgets) {
                budgetListBox.clear();
                for (LabelValueDTO budget : budgets) {
                    budgetListBox.addItem(budget.getLabel(), budget.getValue());
                }
            }
        });
    }

    private void budgetChange() {
        fireOnchange();
    }

    private void fireOnchange() {
        this.changeListeners.fireChange(this);
    }
}
