package fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Widget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SearchParameterDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.ServiceLocator;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.GWTUtils;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.ImageButton;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria.CritereWidget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria.CritereWidgetFactory;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria.CritereLineContainer;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria.InputCritereWidget;
import java.util.ArrayList;
import java.util.List;

/**
 * @author CBO
 */
public class CriteriaTable extends Composite {

    private SearchWidget searchWidget;
    private FlexTable flexTable;

    public CriteriaTable(SearchWidget searchWidget) {
        this.searchWidget = searchWidget;
        this.flexTable = new FlexTable();
        initWidget(this.flexTable);
    }

    public SearchWidget getMainWidget() {
        return searchWidget;
    }

    public void removeAllCriteres() {
        while (getRowCount() > 0) {
            removeRow(getRowCount() - 1);
        }
    }

    public SelectCritereWidget getSelectCritereWidget(int row) {
        return (SelectCritereWidget) getWidget(row, SelectCritereWidget.class);
    }

    public void addEmptyCriterion() {
        final int newCriteriaRowIndex = getRowCount();
        ImageButton removeCriterionButton = new ImageButton(GWTUtils.getContextPath() + "/img/bt_delete.gif", "Supprimer");
        
        AndOrChooserWidget andOrChooserWidget = new AndOrChooserWidget();
        if (newCriteriaRowIndex <= 0) {
            GWTUtils.hide(removeCriterionButton);
            GWTUtils.hide(andOrChooserWidget);
        } else {
            removeCriterionButton.addClickListener(new ClickListener() {

                public void onClick(Widget widget) {
                    removeRow(getRow(widget));
                }
            });
        }
        setWidget(newCriteriaRowIndex, 0, removeCriterionButton);
        setWidget(newCriteriaRowIndex, 1, andOrChooserWidget);
        SelectCritereWidget selectCritereWidget = new SelectCritereWidget(searchWidget.getCritereTypes());
        setWidget(newCriteriaRowIndex, 2, selectCritereWidget);
        selectCritereWidget.addChangeListener(new ChangeListener() {

            public void onChange(Widget widget) {
                initCriteria(getRow(widget));
            }
        });
        initCriteria(newCriteriaRowIndex);
    }

    public boolean checkValidity() {
        // Il faut au moins un champ :
        if (getRowCount() <= 0) {
            Window.alert(new HTML("La recherche doit comporter au moins un crit&egrave;re").getHTML());
            return false;
        }
        // Champs obligatoires :
        for (int row = 0; row < getRowCount(); row++) {
            CritereLineContainer criteriaLineContainer = (CritereLineContainer) getWidget(row, CritereLineContainer.class);
            for (InputCritereWidget inputCritereWidget : criteriaLineContainer.getInputCriteriaWidgets()) {
                if (!inputCritereWidget.isFilled()) {
                    Window.alert(new HTML("Le champ d&eacute;sign&eacute; par le curseur doit &#234;tre rempli.").getHTML());
                    inputCritereWidget.setFocus();
                    return false;
                }
            }
        }
        // Validit des champs :
        for (int row = 0; row < getRowCount(); row++) {
            CritereLineContainer criteriaLineContainer = (CritereLineContainer) getWidget(row, CritereLineContainer.class);
            for (InputCritereWidget inputCritereWidget : criteriaLineContainer.getInputCriteriaWidgets()) {
                try {
                    inputCritereWidget.checkValidity();
                } catch(InputCritereWidget.ValidityException ve) {
                    Window.alert(new HTML(ve.getMessageToDisplay() + ", veuillez corriger le champ champ d&eacute;sign&eacute; par le curseur.").getHTML());
                    inputCritereWidget.setFocus();
                    return false;
                }
            }
        }
        return true;
    }

    private void setWidget(int row, int column, Widget widget) {
        this.flexTable.setWidget(row, column, widget);
    }

    private void removeRow(int row) {
        this.flexTable.removeRow(row);
    }

    private void clearCell(int row, int column) {
        if (this.flexTable.getCellCount(row) > column) {
            this.flexTable.clearCell(row, column);
        }
    }

    private int getRowCount() {
        return this.flexTable.getRowCount();
    }

    private int getRow(Widget widget) {
        for (int i = 0; i < this.flexTable.getRowCount(); i++) {
            for (int j = 0; j < this.flexTable.getCellCount(i); j++) {
                if (widget == this.flexTable.getWidget(i, j)) {
                    return i;
                }
            }
        }
        return -1;
    }

    private void initCriteria(final int row) {
        final SelectCritereWidget selectCritereWidget = getSelectCritereWidget(row);
        clearCell(row, 3);
        ServiceLocator.getXemeliosService().getCritereItems(
                searchWidget.getCollectiviteId(),
                searchWidget.getBudgetId(),
                searchWidget.getDocumentId(),
                searchWidget.getEtatId(),
                searchWidget.getElementId(),
                selectCritereWidget.getSelectedCritere().getId(),
                new AsyncCallback<List<CritereItemDTO>>() {

                    public void onFailure(Throwable throwable) {
                        GWTUtils.handleRPCError(throwable);
                    }

                    public void onSuccess(List<CritereItemDTO> result) {
                        if (result != null && !result.isEmpty()) {
                            CritereLineContainer criteriaLineContainer = new CritereLineContainer(CriteriaTable.this, selectCritereWidget.getSelectedCritere().getId());
                            for (CritereItemDTO critereItem : result) {
                                CritereWidget cw = CritereWidgetFactory.createWidget(criteriaLineContainer, critereItem);
                                if (cw != null) {
                                    criteriaLineContainer.add(cw);    
                                }
                            }
                            setWidget(row, 3, criteriaLineContainer);
                        }
                    }
                });
    }

    public List<SearchParameterDTO> getSearchParameters() {
        List<SearchParameterDTO> toReturn = new ArrayList<SearchParameterDTO>();
        for (int i = 0; i < getRowCount(); i++) {
            toReturn.add(getSearchParameter(i));
        }
        return toReturn;
    }

    private SearchParameterDTO getSearchParameter(int row) {
        SearchParameterDTO toReturn = new SearchParameterDTO();
        AndOrChooserWidget andOrChooserWidget = (AndOrChooserWidget)getWidget(row, AndOrChooserWidget.class);
        if (andOrChooserWidget != null && row != 0) {
            toReturn.setOperator(andOrChooserWidget.getSelectedValue());
        }
        toReturn.setCritereModelId(getSelectCritereWidget(row).getSelectedCritere().getId());
        CritereLineContainer criteriaLineContainer = (CritereLineContainer) getWidget(row, CritereLineContainer.class);
        for (InputCritereWidget inputCritereWidget : criteriaLineContainer.getInputCriteriaWidgets()) {
            toReturn.putInputValue(inputCritereWidget.getInputId(), inputCritereWidget.getInputValue());
        }
        return toReturn;
    }

    private Widget getWidget(int row, Class clazz) {
        for (int i = 0; i < this.flexTable.getCellCount(row); i++) {
            Widget widget = this.flexTable.getWidget(row, i);
            if (widget != null && widget.getClass().equals(clazz)) {
                return widget;
            }
        }
        return null;
    }
}
