package fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalSplitPanel;
import com.google.gwt.user.client.ui.Widget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereTypeDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.InitParametersDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SearchResultDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.ServiceLocator;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.DoneListener;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.GWTUtils;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.PaginationWidget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.ButtonsBarWidget.ClickListenerType;
import java.util.List;

/**
 * @author CBO
 * Widget principal (permettant la recherche)
 */
public class SearchWidget extends Composite {

    private String documentId;
    private String etatId;
    private String elementId;
    private InitParametersDTO initParameters;
    private DockPanel mainPanel;
    private ColAndBudChooserWidget colAndBudChooserWidget;
    private ButtonsBarWidget buttonsBarWidget;
    private VerticalSplitPanel verticalSplitPanel;
    private CriteriaTable criteriaTable;
    private HTML resultTableContainer;
    private StatusBar statusBar;

    public SearchWidget(String documentId, String etatId, String elementId) {
        this.documentId = documentId;
        this.etatId = etatId;
        this.elementId = elementId;
        this.mainPanel = new DockPanel();
        this.mainPanel.setSize("100%", "100%");
        this.buttonsBarWidget = new ButtonsBarWidget();
        this.mainPanel.add(this.buttonsBarWidget, DockPanel.WEST);
        this.colAndBudChooserWidget = new ColAndBudChooserWidget(this);
        this.mainPanel.add(this.colAndBudChooserWidget, DockPanel.NORTH);
        //this.statusBar = new StatusBar();
        //this.mainPanel.add(this.statusBar, DockPanel.SOUTH);
        this.mainPanel.add(new Label("Status bar"), DockPanel.SOUTH);
        this.verticalSplitPanel = new VerticalSplitPanel();
        this.criteriaTable = new CriteriaTable(this);
        this.verticalSplitPanel.setTopWidget(criteriaTable);
        this.resultTableContainer = new HTML();
        this.resultTableContainer.getElement().setId("resultTableContainer");
        this.verticalSplitPanel.setBottomWidget(resultTableContainer);
        this.verticalSplitPanel.setSplitPosition("87px");
        this.verticalSplitPanel.setSize("100%", "100%");
        this.mainPanel.add(this.verticalSplitPanel, DockPanel.CENTER);
        this.mainPanel.setCellWidth(this.verticalSplitPanel, "100%");
        this.mainPanel.setCellHeight(this.verticalSplitPanel, "100%");
        initWidget(this.mainPanel);
    }

    /**
     * Mthode permettant de charger les donnes d'initialisation de la recherche
     */
    public void init(final DoneListener doneListener) {
        ServiceLocator.getXemeliosService().getCWInitParameters(documentId, etatId, elementId, new AsyncCallback<InitParametersDTO>() {

            public void onFailure(Throwable throwable) {
                GWTUtils.handleRPCError(throwable);
            }

            public void onSuccess(InitParametersDTO result) {
                SearchWidget.this.initParameters = result;
                colAndBudChooserWidget.init(result.getCollectivites(), result.getFirstCollectiviteBudgets());
                colAndBudChooserWidget.addChangeListener(new ChangeListener() {

                    public void onChange(Widget arg0) {
                        collectiviteOrBudgetChange();
                    }
                });
                buttonsBarWidget.addClickListener(ClickListenerType.ADD_CRITERE, new ClickListener() {

                    public void onClick(Widget arg0) {
                        addCritere();
                    }
                });
                buttonsBarWidget.addClickListener(ClickListenerType.SEARCH, new ClickListener() {

                    public void onClick(Widget arg0) {
                        search();
                    }
                });
                buttonsBarWidget.addClickListener(ClickListenerType.RESET_CRITERES, new ClickListener() {

                    public void onClick(Widget arg0) {
                        resetCriteres();
                    }
                });
                addCritere();
                doneListener.done(result);
            }
        });
    }

    public void addCritere() {
        this.criteriaTable.addEmptyCriterion();
    }

    public void resetCriteres() {
        this.criteriaTable.removeAllCriteres();
        this.criteriaTable.addEmptyCriterion();
    }

    public void search() {
        if (criteriaTable.checkValidity()) {
            startSearch();
            ServiceLocator.getXemeliosService().search(getCollectiviteId(), getBudgetId(), getDocumentId(), getEtatId(), getElementId(), criteriaTable.getSearchParameters(), new AsyncCallback<SearchResultDTO>() {

                public void onFailure(Throwable throwable) {
                    GWTUtils.handleRPCError(throwable);
                }

                public void onSuccess(SearchResultDTO result) {
                    handleSearchResult(result);
                }
            });

        }
    }

    private void startSearch() {
        resultTableContainer.setHTML("<center><img src=\"" + GWTUtils.getContextPath() + "/img/searching.gif\"/></center>");
    }

    private void handleSearchResult(SearchResultDTO result) {
        resultTableContainer.setHTML(transformXml(result.getXml()));
    }

    private void collectiviteOrBudgetChange() {
        resetCriteres();
    }

    public String getDocumentId() {
        return this.documentId;
    }

    public String getElementId() {
        return this.elementId;
    }

    public String getEtatId() {
        return this.etatId;
    }

    public String getCollectiviteId() {
        return this.colAndBudChooserWidget.getCollectiviteId();
    }

    public String getBudgetId() {
        return this.colAndBudChooserWidget.getBudgetId();
    }

    public List<CritereTypeDTO> getCritereTypes() {
        return this.initParameters.getCritereTypes();
    }

    private native String transformXml(String searchResultAsXml)/*-{
    return $wnd.transformXml(searchResultAsXml);
    }-*/;
}
