package fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria;

import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereItemVisitor;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.DateCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.TextCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.LabelCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.NumericCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SelectOneCritereItemDTO;
import java.util.HashMap;
import java.util.Map;

/**
 * @author CBO
 * Fabrique permettant de crer les composants graphiques affichs pour la saisie d'un critre de recherche  partir des DTO
 */
public class CritereWidgetFactory {

    static public CritereWidget createWidget(CritereLineContainer criteriaLineContainer, CritereItemDTO critereItem) {
        CriteriaWidgetFactoryCritereItemVisitor criteriaWidgetFactoryCritereItemVisitor = new CriteriaWidgetFactoryCritereItemVisitor(criteriaLineContainer);
        critereItem.accept(criteriaWidgetFactoryCritereItemVisitor);
        return criteriaWidgetFactoryCritereItemVisitor.toReturn;
    }

    static public class InputCritereWidgetRepository {

        private Map<String, InputCritereWidget> widgetByInputId = new HashMap<String, InputCritereWidget>();

        public void add(InputCritereWidget inputCritereWidget) {
            widgetByInputId.put(inputCritereWidget.getInputId(), inputCritereWidget);
        }

        public InputCritereWidget getWidget(String inputId) {
            return widgetByInputId.get(inputId);
        }
    }

    static private class CriteriaWidgetFactoryCritereItemVisitor implements CritereItemVisitor {

        private CritereLineContainer critereLineContainer;
        private CritereWidget toReturn;

        public CriteriaWidgetFactoryCritereItemVisitor(CritereLineContainer critereLineContainer) {
            this.critereLineContainer = critereLineContainer;
        }

        public void visit(LabelCritereItemDTO labelCritereItem) {
            this.toReturn = new LabelCritereWidget(critereLineContainer, labelCritereItem.getLibelle());
        }

        public void visit(final SelectOneCritereItemDTO selectOneCritereItem) {
            this.toReturn = new SelectOneCritereWidget(critereLineContainer, selectOneCritereItem);
        }

        public void visit(TextCritereItemDTO inputTextCritereItem) {
            this.toReturn = new TextCritereWidget(critereLineContainer, inputTextCritereItem);
        }

        public void visit(DateCritereItemDTO inputDateCritereItem) {
            this.toReturn = new DateCritereWidget(critereLineContainer, inputDateCritereItem);
        }
        
        public void visit(NumericCritereItemDTO numericCritereItem) {
            this.toReturn = new NumericCritereWidget(critereLineContainer, numericCritereItem);
        }
    }

}
