package fr.gouv.finances.dgfip.xemelios.web.gwt.client.widget.search.criteria;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.InputCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.LabelValueDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SelectOneCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.ServiceLocator;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.GWTUtils;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author CBO
 * Permet de choisir une valeur parmi plusieurs
 */
public class SelectOneCritereWidget extends InputCritereWidget {

    private SelectOneCritereItemDTO selectOneCritereItem;
    private ListBox listBox;

    public SelectOneCritereWidget(final CritereLineContainer critereLineContainer, final SelectOneCritereItemDTO selectOneCritereItem) {
        super(critereLineContainer, selectOneCritereItem);
        this.selectOneCritereItem = selectOneCritereItem;
        this.listBox = new ListBox();
        fillListBox(selectOneCritereItem.getItems(), selectOneCritereItem.getSelectedItem());
        if (!selectOneCritereItem.getCriteresDependants().isEmpty()) {
            addChangeListener(new ChangeListener() {

                public void onChange(Widget widget) {
                    for (InputCritereItemDTO inputCritereItemDTO : selectOneCritereItem.getCriteresDependants()) {
                        InputCritereWidget dependantInputCritereWidget = critereLineContainer.getInputCritereWidget(inputCritereItemDTO.getInputId());
                        dependantInputCritereWidget.refresh();
                    }

                }
            });
        }
        initWidget(listBox);
    }

    private void updateListBoxContent(List<LabelValueDTO> items, LabelValueDTO selectedItem) {
        selectOneCritereItem.setItems(items);
        selectOneCritereItem.setSelectedItem(selectedItem);
        fillListBox(items, selectedItem);
    }

    public void fillListBox(Collection<LabelValueDTO> labelValues, LabelValueDTO selectedItem) {
        listBox.clear();
        int index = 0;
        for (LabelValueDTO labelValue : labelValues) {
            listBox.addItem(labelValue.getLabel(), labelValue.getValue());
            if (labelValue.equals(selectOneCritereItem.getSelectedItem())) {
                listBox.setSelectedIndex(index);
            }
            index++;
        }
    }

    public void addChangeListener(ChangeListener changeListener) {
        listBox.addChangeListener(changeListener);
    }

    @Override
    public void checkValidity() throws ValidityException {
        
    }

    @Override
    public void setFocus() {
        this.listBox.setFocus(true);
    }

    @Override
    public String getInputId() {
        return this.selectOneCritereItem.getInputId();
    }

    @Override
    public String getInputValue() {
        return listBox.getValue(listBox.getSelectedIndex());
    }

    @Override
    public void refresh() {
        Map<String, String> inputValues = new HashMap<String, String>();
        for (InputCritereItemDTO criteresMaitre : selectOneCritereItem.getCriteresMaitres()) {
            InputCritereWidget inputWidget = getCritereLineContainer().getInputCritereWidget(criteresMaitre.getInputId());
            inputValues.put(inputWidget.getInputId(), inputWidget.getInputValue());
            ServiceLocator.getXemeliosService().getUpdatedSelectOneCritereItemDTO(
                    getCritereLineContainer().getCriteriaTable().getMainWidget().getCollectiviteId(),
                    getCritereLineContainer().getCriteriaTable().getMainWidget().getBudgetId(),
                    getCritereLineContainer().getCriteriaTable().getMainWidget().getDocumentId(),
                    getCritereLineContainer().getCriteriaTable().getMainWidget().getEtatId(),
                    getCritereLineContainer().getCriteriaTable().getMainWidget().getElementId(),
                    getCritereLineContainer().getCritereId(),
                    getInputId(),
                    inputValues,
                    new AsyncCallback<SelectOneCritereItemDTO>() {

                        public void onFailure(Throwable throwable) {
                            GWTUtils.handleRPCError(throwable);
                        }

                        public void onSuccess(SelectOneCritereItemDTO result) {
                            if (result != null) {
                                updateListBoxContent(result.getItems(), result.getSelectedItem());
                            }
                        }
                    });
        }
    }
}
