package fr.gouv.finances.dgfip.xemelios.web.gwt.server.service;

import com.google.gwt.user.client.rpc.IncompatibleRemoteServiceException;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.dgfip.utils.Pair;
import fr.gouv.finances.dgfip.xemelios.common.config.ChampModel;
import fr.gouv.finances.dgfip.xemelios.common.config.CritereModel;
import fr.gouv.finances.dgfip.xemelios.common.config.ElementModel;
import fr.gouv.finances.dgfip.xemelios.common.config.HiddenModel;
import fr.gouv.finances.dgfip.xemelios.common.config.InputModel;
import fr.gouv.finances.dgfip.xemelios.common.config.ListeResultatModel;
import fr.gouv.finances.dgfip.xemelios.common.config.OptionModel;
import fr.gouv.finances.dgfip.xemelios.common.config.PropertyModel;
import fr.gouv.finances.dgfip.xemelios.common.config.RecherchePaireModel;
import fr.gouv.finances.dgfip.xemelios.common.config.SelectModel;
import fr.gouv.finances.dgfip.xemelios.common.config.SpecialKeyEntryModel;
import fr.gouv.finances.dgfip.xemelios.common.config.WidgetModel;
import fr.gouv.finances.dgfip.xemelios.data.DataAccessException;
import fr.gouv.finances.dgfip.xemelios.data.DataConfigurationException;
import fr.gouv.finances.dgfip.xemelios.data.DataHandler;
import fr.gouv.finances.dgfip.xemelios.data.DataLayerManager;
import fr.gouv.finances.dgfip.xemelios.data.DataResultSet;
import fr.gouv.finances.dgfip.xemelios.services.XemeliosServiceFacade;
import fr.gouv.finances.dgfip.xemelios.web.AttributeHelper;
import fr.gouv.finances.dgfip.xemelios.web.WebAppCst;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.InitParametersDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereTypeDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.CritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.InputCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.DateCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.TextCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.LabelCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.LabelValueDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.NumericCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SearchParameterDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SearchResultDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.dto.SelectOneCritereItemDTO;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.SessionExpiredException;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.service.XemeliosService;
import fr.gouv.finances.dgfip.xemelios.web.gwt.client.utils.GWTUtils;
import fr.gouv.finances.dgfip.xemelios.web.utils.SimpleXmlWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.xml.sax.SAXException;

/**
 * @author CBO
 */
public class XemeliosServiceImpl extends RemoteServiceServlet implements XemeliosService {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(XemeliosServiceImpl.class);

    @Override
    protected void doUnexpectedFailure(Throwable e) {
        try {
            throw e;
        } catch (SessionExpiredException see) {
            ActionMessages actionMessages = new ActionMessages();
            actionMessages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("connexion.sessionExpiree"));
            getThreadLocalRequest().getSession().setAttribute(Globals.ERROR_KEY, actionMessages);
            writeResponseForClientRedirection(getServletContext(), getThreadLocalResponse(), getThreadLocalRequest().getContextPath());
        } catch(IncompatibleRemoteServiceException irse) {
          writeResponseForClientRedirection(getServletContext(), getThreadLocalResponse(), getThreadLocalRequest().getContextPath() + "/gwtOutOfDate.jsp");
        } catch (Throwable t) {
            getThreadLocalRequest().getSession().setAttribute(WebAppCst.SessionAttributes.ajaxExceptionKey, t);
            writeResponseForClientRedirection(getServletContext(), getThreadLocalResponse(), getThreadLocalRequest().getContextPath() + "/erreur.jsp");
        }
    }

    private void writeResponseForClientRedirection(ServletContext servletContext, HttpServletResponse response, String url) {
        try {
            response.setContentType("text/plain");
            response.setStatus(GWTUtils.REMOTE_ERROR_FOR_REDIRECTION_CODE);
            response.getWriter().write(url);
        } catch (IOException ex) {
            servletContext.log("respondWithUnexpectedFailure failed while sending the previous failure to the client", ex);
        }
    }

    @Override
    protected void onBeforeRequestDeserialized(String serializedRequest) {
        if (AttributeHelper.getPrincipal(getThreadLocalRequest().getSession()) == null) {
            throw new SessionExpiredException();
        }
    }

    /**
     * Retourne les parametres ncessaires  l'initialisation du widget d'laboration des critres de recherche
     */
    public InitParametersDTO getCWInitParameters(String documentId, String etatId, String elementId) {
        List<LabelValueDTO> collectivites = getCollectivites(documentId);
        List<LabelValueDTO> budgetsForFirstCollectivite = null;
        if (!collectivites.isEmpty()) {
            budgetsForFirstCollectivite = getBudgets(documentId, collectivites.iterator().next().getValue());
        } else {
            budgetsForFirstCollectivite = new ArrayList<LabelValueDTO>();
        }
        return new InitParametersDTO(
                collectivites,
                budgetsForFirstCollectivite,
                getCriteres(documentId, etatId, elementId));
    }

    private List<LabelValueDTO> getCollectivites(String documentId) {
        ArrayList<LabelValueDTO> toReturn = new ArrayList<LabelValueDTO>();
        try {
            Vector<Pair> pairs = DataLayerManager.getImplementation().getCollectivites(XemeliosServiceFacade.getDocumentsModel(documentId));
            for (Pair pair : pairs) {
                toReturn.add(new LabelValueDTO(pair.key, pair.libelle));
            }
            return toReturn;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public List<LabelValueDTO> getBudgets(String documentId, String collectiviteId) {
        ArrayList<LabelValueDTO> toReturn = new ArrayList<LabelValueDTO>();
        try {
            Vector<Pair> pairs = DataLayerManager.getImplementation().getBudgets(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null));
            for (Pair pair : pairs) {
                toReturn.add(new LabelValueDTO(pair.key, pair.libelle));
            }
            return toReturn;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public List<CritereTypeDTO> getCriteres(String documentId, String etatId, String elementId) {
        List<CritereTypeDTO> toReturn = new ArrayList<CritereTypeDTO>();
        for (CritereModel cm : XemeliosServiceFacade.getDocumentsModel(documentId).getEtatById(etatId).getElementById(elementId).getAvailableCriteres()) {
            toReturn.add(new CritereTypeDTO(cm.getId(), cm.getLibelle()));
        }
        return toReturn;
    }

    public SelectOneCritereItemDTO getUpdatedSelectOneCritereItemDTO(String collectiviteId, String budgetId, String documentId, String etatId, String elementId, String critereId, String selectId, Map<String, String> inputValuesById) {
        String smId = selectId.substring(selectId.indexOf(":") + 1);
        CritereModel critereModel = XemeliosServiceFacade.getDocumentsModel(documentId).getEtatById(etatId).getElementById(elementId).getCritere(critereId);
        for (XmlMarshallable xm : critereModel.getInputs()) {
            if (xm instanceof SelectModel) {
                SelectModel sm = (SelectModel) xm;
                if (sm.getId().equals(smId)) {
                    // On a trouv le composant :
                    Collection<Pair> data = null;
                    data = sm.getNewRecipient();
                    if (sm.getOptions() != null) {
                        for (OptionModel om : sm.getOptions()) {
                            data.add(new Pair(om.getValue(), om.getLibelle()));
                        }
                    }
                    for (RecherchePaireModel rpm : sm.getRecherches()) {
                        if (rpm.getPath() != null) {
                            // on fait une requete dans le referentiel pour remplir la liste
                            try {
                                Collection<Pair> localData = null;
                                localData = rpm.getNewRecipient();
                                localData.addAll(DataLayerManager.getImplementation().queryRepository(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null), rpm));
                                data.addAll(localData);
                            } catch (Exception dbEx) {
                                throw new RuntimeException(dbEx);
                            }
                        }
                    }
                    // Il y a des dpendances, on recherche les ##VALEUR:id##
                    for (SpecialKeyEntryModel skem : sm.getSpecialKeys()) {
                        if (skem.isDepends()) {
                            String path = skem.getKey1().getData();
                            String itemName = path.substring(2, path.length() - 2);
                            Collection<Pair> tmp = sm.getNewRecipient();
                            try {
                                if (skem.getPos() == 2) {
                                    tmp.addAll(DataLayerManager.getImplementation().getSpecialKeys2(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null), new Pair(budgetId, null), new Pair(inputValuesById.get(itemName), inputValuesById.get(itemName)), skem.isDistinct()));
                                } else {
                                    throw new RuntimeException("Not yet implemented");
                                /*id = cv.inputs.get(itemName);
                                if (id.getInputValue() != null && id.getInputValue().length() > 0) {
                                key2 = id.getInputValue();
                                }
                                tmp.addAll(DataLayerManager.getImplementation().getSpecialKeys3(dm,collectivite,budget,new Pair(key1,key1),new Pair(key2,key2),skem.isDistinct()));*/
                                }
                            } catch (DataConfigurationException ex) {
                                throw new RuntimeException(ex);
                            } catch (DataAccessException ex) {
                                throw new RuntimeException(ex);
                            }
                            data.addAll(tmp);
                        }
                    }
                    // Il y a des dpendances, on recherche les ##VALEUR:id##
                    for (SpecialKeyEntryModel skem : sm.getSpecialKeys()) {
                        if (!skem.isDepends()) {// Si pas de dependance, on peut remplir maintenant :
                            try {
                                data.addAll(DataLayerManager.getImplementation().getSpecialKeys1(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null), new Pair(budgetId, null), skem.isDistinct()));
                            } catch (Exception dbEx) {
                                throw new RuntimeException(dbEx);
                            }
                        }
                    }
                    SelectOneCritereItemDTO selectOneCritereItem = new SelectOneCritereItemDTO("VALEUR:" + sm.getId());
                    for (Pair pair : data) {
                        LabelValueDTO labelValue = new LabelValueDTO(pair.key, pair.libelle);
                        selectOneCritereItem.addItem(labelValue);
                        if (sm.getValue() != null && sm.getValue().equals(pair.key)) {
                            selectOneCritereItem.setSelectedItem(labelValue);
                        }
                    }
                    return selectOneCritereItem;
                }
            }
        }
        return null;
    }

    public List<CritereItemDTO> getCritereItems(String collectiviteId, String budgetId, String documentId, String etatId, String elementId, String critereId) {
        Map<String, SelectOneCritereItemDTO> selectOneCritereItemByKey1 = new HashMap<String, SelectOneCritereItemDTO>();
        List<CritereItemDTO> toReturn = new ArrayList<CritereItemDTO>();
        CritereModel critereModel = XemeliosServiceFacade.getDocumentsModel(documentId).getEtatById(etatId).getElementById(elementId).getCritere(critereId);
        for (XmlMarshallable xm : critereModel.getInputs()) {
            if (xm instanceof InputModel) {
                InputModel im = (InputModel) xm;
                if (im.getLibelle() != null) {
                    LabelCritereItemDTO labelCritereItem = new LabelCritereItemDTO();
                    labelCritereItem.setLibelle(im.getLibelle());
                    toReturn.add(labelCritereItem);
                }
                String datatype = im.getDatatype();
                if (datatype != null) {
                    Pair pairs[] = null;
                    try {
                        pairs = DataLayerManager.getImplementation().getOperators(datatype);
                        if (pairs.length > 0) {
                            SelectOneCritereItemDTO operatorsSelect = new SelectOneCritereItemDTO("OPERATEUR:" + im.getId());
                            for (Pair pair : pairs) {
                                operatorsSelect.addItem(new LabelValueDTO(pair.key, pair.libelle));
                            }
                            toReturn.add(operatorsSelect);
                            InputCritereItemDTO input = null;
                            if ("date".equals(im.getDatatype())) {
                                input = new DateCritereItemDTO("VALEUR:" + im.getId(), im.getXmlDateFormat());
                            } else if ("numeric".equals(im.getDatatype())) {
                                input = new NumericCritereItemDTO("VALEUR:" + im.getId());
                            } else {
                                input = new TextCritereItemDTO("VALEUR:" + im.getId());
                                ((TextCritereItemDTO) input).setUppercase(im.isUppercase());
                            }
                            toReturn.add(input);
                        }

                    } catch (Exception dbEx) {
                        throw new RuntimeException(dbEx);
                    }
                }
            } else if (xm instanceof SelectModel) {
                SelectModel sm = (SelectModel) xm;
                if (sm.getLibelle() != null) {
                    LabelCritereItemDTO labelCritereItem = new LabelCritereItemDTO();
                    labelCritereItem.setLibelle(sm.getLibelle());
                    toReturn.add(labelCritereItem);
                }
                Collection<Pair> data = null;
                data = sm.getNewRecipient();
                if (sm.getOptions() != null) {
                    for (OptionModel om : sm.getOptions()) {
                        data.add(new Pair(om.getValue(), om.getLibelle()));
                    }
                }
                for (RecherchePaireModel rpm : sm.getRecherches()) {
                    if (rpm.getPath() != null) {
                        // on fait une requete dans le referentiel pour remplir la liste
                        try {
                            Collection<Pair> localData = null;
                            localData = rpm.getNewRecipient();
                            localData.addAll(DataLayerManager.getImplementation().queryRepository(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null), rpm));
                            data.addAll(localData);
                        } catch (Exception dbEx) {
                            throw new RuntimeException(dbEx);
                        }
                    }
                }

                for (SpecialKeyEntryModel skem : sm.getSpecialKeys()) {
                    if (!skem.isDepends()) {
                        // Si pas de dependance, on peut remplir maintenant :
                        try {
                            data.addAll(DataLayerManager.getImplementation().getSpecialKeys1(XemeliosServiceFacade.getDocumentsModel(documentId), new Pair(collectiviteId, null), new Pair(budgetId, null), skem.isDistinct()));
                        } catch (Exception dbEx) {
                            throw new RuntimeException(dbEx);
                        }
                    }
                }
                SelectOneCritereItemDTO selectOneCritereItem = new SelectOneCritereItemDTO("VALEUR:" + sm.getId());
                LabelValueDTO firstLabelValue = null;
                int idx = 0;
                for (Pair pair : data) {
                    LabelValueDTO labelValue = new LabelValueDTO(pair.key, pair.libelle);
                    if (idx == 0) {
                        firstLabelValue = labelValue;
                    }
                    idx++;
                    selectOneCritereItem.addItem(labelValue);
                    if (sm.getValue() != null && sm.getValue().equals(pair.key)) {
                        selectOneCritereItem.setSelectedItem(labelValue);
                    }
                }
                if (selectOneCritereItem.getSelectedItem() == null && firstLabelValue != null) {
                    selectOneCritereItem.setSelectedItem(firstLabelValue);
                }
                toReturn.add(selectOneCritereItem);
                selectOneCritereItemByKey1.put("##VALEUR:" + sm.getId() + "##", selectOneCritereItem);
            }
        }
        // Dfinition des dpendances entre les champs :
        for (XmlMarshallable xm : critereModel.getInputs()) {
            if (xm instanceof SelectModel) {
                SelectModel sm = (SelectModel) xm;
                for (SpecialKeyEntryModel skem : sm.getSpecialKeys()) {
                    if (skem.isDepends()) {
                        SelectOneCritereItemDTO masterS = selectOneCritereItemByKey1.get(skem.getKey1().getData());
                        InputCritereItemDTO child = selectOneCritereItemByKey1.get("##VALEUR:" + sm.getId() + "##");
                        masterS.addCritereDependant(child);
                        child.addCritereMaitre(masterS);
                    }
                }
            }
        }
        // Dfinition des valeurs des champs dpendants :
        for (XmlMarshallable xm : critereModel.getInputs()) {
            if (xm instanceof SelectModel) {
                SelectModel sm = (SelectModel) xm;
                for (SpecialKeyEntryModel skem : sm.getSpecialKeys()) {
                    if (skem.isDepends()) {
                        SelectOneCritereItemDTO child = selectOneCritereItemByKey1.get("##VALEUR:" + sm.getId() + "##");
                        Map<String, String> inputValuesById = new HashMap<String, String>();
                        for (InputCritereItemDTO inputCritereItemDTO : child.getCriteresMaitres()) {
                            if (inputCritereItemDTO instanceof SelectOneCritereItemDTO) {
                                SelectOneCritereItemDTO masterS = (SelectOneCritereItemDTO)inputCritereItemDTO;
                                if (masterS.getSelectedItem() != null) {
                                    inputValuesById.put(inputCritereItemDTO.getInputId(), masterS.getSelectedItem().getValue());
                                }
                            }
                        }
                        SelectOneCritereItemDTO updatedSelectItems = getUpdatedSelectOneCritereItemDTO(collectiviteId, budgetId, documentId, etatId, elementId, critereId, child.getInputId(), inputValuesById);
                        if (updatedSelectItems != null) {
                            child.setItems(updatedSelectItems.getItems());
                            child.setSelectedItem(updatedSelectItems.getSelectedItem());
                        }
                    }
                }
            }
        }
        return toReturn;
    }

    public SearchResultDTO search(String collectiviteId, String budgetId, String documentId, String etatId, String elementId, List<SearchParameterDTO> searchParams) {
        try {
            StringBuffer xpathQuery = new StringBuffer();
            ElementModel elementModel = XemeliosServiceFacade.getDocumentsModel(documentId).getEtatById(etatId).getElementById(elementId);
            xpathQuery.append(elementModel.getPath().getPath());
            Vector<CritereModel> critereModels = new Vector<CritereModel>();
            for (SearchParameterDTO searchParam : searchParams) {
                processSearchParameter(elementModel, searchParam, critereModels, xpathQuery);
            }
            ListeResultatModel lrm = new ListeResultatModel(ListeResultatModel.QN);
            lrm.setParent(elementModel);
            // TODO champs optionnels
            ListeResultatModel lrm2 = elementModel.getListeResultat().clone();
            for (ChampModel cm : lrm2.getListeChamps()) {
                if (cm.isAfficheDefaut() && cm.isAffichable()) {
                    lrm.addChild(cm, ChampModel.QN);
                }
            }
            lrm.setTriDefaut(lrm2.getTriDefaut());
            lrm.setOrdre(lrm2.getOrdre());
            for (HiddenModel hm : lrm2.getHiddens()) {
                lrm.addChild(hm, HiddenModel.QN);
            }
            for (WidgetModel wm : lrm2.getWidgets()) {
                lrm.addChild(wm, WidgetModel.QN);
            }
            DataResultSet dataResultSet = DataLayerManager.getImplementation().search(
                    elementModel,
                    new Pair(collectiviteId, collectiviteId),
                    new Pair(budgetId, budgetId),
                    xpathQuery.toString(),
                    lrm,
                    critereModels);
            getThreadLocalRequest().getSession().setAttribute("dataResultSet", dataResultSet);
            return dataResultToSearchResult(dataResultSet, lrm);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    private SearchResultDTO dataResultToSearchResult(DataResultSet dataResultSet, ListeResultatModel lrm) throws IOException, ParseException {
        StringWriter stringWriter = new StringWriter();
        SimpleXmlWriter simpleXmlWriter = new SimpleXmlWriter(stringWriter);
        simpleXmlWriter.startElement("page");
        simpleXmlWriter.startElement("columns");
        for (ChampModel champModel : lrm.getListeChamps()) {
            simpleXmlWriter.writeEmptyElement("column",
                    new SimpleXmlWriter.Attribute("id", champModel.getId()),
                    new SimpleXmlWriter.Attribute("libelle", champModel.getLibelle()),
                    new SimpleXmlWriter.Attribute("datatype", champModel.getDatatype()));
        }
        simpleXmlWriter.endElement("columns");
        simpleXmlWriter.startElement("values");
        while (dataResultSet.hasNext()) {
            simpleXmlWriter.startElement("line");
            DataHandler dataHandler = dataResultSet.next();
            int i = 0;
            for (ChampModel champModel : lrm.getListeChamps()) {
                Object value = dataHandler.getValueAtColumn(i);
                simpleXmlWriter.writeTextElement("cell", value != null ? value.toString() : null);
                i++;
            }
            simpleXmlWriter.endElement("line");
        }
        simpleXmlWriter.endElement("values");
        simpleXmlWriter.endElement("page");
        simpleXmlWriter.flush();
        SearchResultDTO toReturn = new SearchResultDTO();
        toReturn.setXml(stringWriter.toString());
        return toReturn;
    }

    private void processSearchParameter(ElementModel elementModel, SearchParameterDTO searchParam, Vector<CritereModel> critereModels, StringBuffer xpathQuery) throws SAXException {
        CritereModel critereModel = elementModel.getCritere(searchParam.getCritereModelId()).clone();
        if (searchParam.getOperator() != null) {
            critereModel.addChild(new PropertyModel("AND_OR", searchParam.getOperator()), PropertyModel.QN);
            if ("||".equals(searchParam.getOperator())) {
                xpathQuery.append("|").append(elementModel.getPath().getPath());
            }
        }
        String data = critereModel.getPath().getPath();
        int pos = data.indexOf("##");
        while (pos >= 0) {
            int endPos = data.indexOf("##", pos + 2);
            String pattern = data.substring(pos, endPos + 2);
            String paramName = pattern.substring(2, pattern.length() - 2);
            String paramValue = searchParam.getValue(paramName);
            if (paramValue != null) {
                data = data.replace(pattern, paramValue);
            } else {
                data = data.replace(pattern, "\"\"");
            }
            pos = data.indexOf("##");
            critereModel.addChild(new PropertyModel(paramName, paramValue), PropertyModel.QN);
        }
        data = data.replaceAll("\\{", "[").replaceAll("\\}", "]");
        critereModels.add(critereModel);
        xpathQuery.append(data);
    }
}
