package fr.gouv.finances.dgfip.xemelios.web.utils.filters;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 * @author CBO
 * Filtre permettant d'ignorer les headers de type "no cache"
 */
public class IgnoreNoCacheHeadersFilter implements Filter {

  public void init(FilterConfig filterConfig) throws ServletException {
  }

  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
    if ((response instanceof HttpServletResponse) && ! (response instanceof IgnoreNoCacheHeaderResponseWrapper)
            && "true".equalsIgnoreCase(request.getParameter("ignore_no_cache"))) {
      response = new IgnoreNoCacheHeaderResponseWrapper((HttpServletResponse)response);
    }
    chain.doFilter(request, response);
  }

  public void destroy() {
  }

  static public class IgnoreNoCacheHeaderResponseWrapper extends HttpServletResponseWrapper {

    public IgnoreNoCacheHeaderResponseWrapper(HttpServletResponse response) {
      super(response);
    }

    @Override
    public void addHeader(String name, String value) {
      if (!"Cache-Control".equalsIgnoreCase(name) && !"Expires".equalsIgnoreCase(name) && !("Pragma".equalsIgnoreCase(name) && "No-cache".equalsIgnoreCase(value))) {
        super.addHeader(name, value);
      }
    }

    @Override
    public void setHeader(String name, String value) {
      if (!"Cache-Control".equalsIgnoreCase(name) && !"Expires".equalsIgnoreCase(name) && !("Pragma".equalsIgnoreCase(name) && "No-cache".equalsIgnoreCase(value))) {
        super.setHeader(name, value);
      }
    }

    @Override
    public void addDateHeader(String name, long value) {
      if (!"Cache-Control".equalsIgnoreCase(name) && !"Expires".equalsIgnoreCase(name)) {
        super.addDateHeader(name, value);
      }
    }

    @Override
    public void addIntHeader(String name, int value) {
      if (!"Cache-Control".equalsIgnoreCase(name) && !"Expires".equalsIgnoreCase(name)) {
        super.addIntHeader(name, value);
      }
    }

    @Override
    public void setDateHeader(String name, long value) {
      if (!"Cache-Control".equalsIgnoreCase(name) && !"Expires".equalsIgnoreCase(name)) {
        super.setDateHeader(name, value);
      }
    }
  }
}
