package fr.gouv.finances.dgfip.xemelios.web.utils.filters;

import fr.gouv.finances.dgfip.xemelios.web.*;
import fr.gouv.finances.dgfip.xemelios.web.AttributeHelper;
import fr.gouv.finances.dgfip.xemelios.web.Principal;
import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import org.apache.log4j.MDC;

/**
 * @author CBO
 */
public class Log4jMDCServletFilter implements Filter {

  static private long conversationId = 0;

  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
    try {
      if (MDC.get("user") == null) {
        MDC.put("user", getUser(request));
      }
      if (MDC.get("ip") == null) {
        MDC.put("ip", getIP(request));
      }
      if (MDC.get("convId") == null) {
        MDC.put("convId", getConversationId());
      }
      chain.doFilter(request, response);
    } finally {
      MDC.remove("user");
      MDC.remove("ip");
      MDC.remove("convId");
    }
  }

  private HttpSession getNullableSession(ServletRequest request) {
    if (request instanceof HttpServletRequest) {
      return ((HttpServletRequest) request).getSession(false);
    } else {
      return null;
    }
  }

  private synchronized String getConversationId() {
    return "" + conversationId++;
  }

  private String getUser(ServletRequest request) {
    HttpSession session = getNullableSession(request);
    if (session != null) {
      Principal principal = AttributeHelper.getPrincipal(session);
      if (principal != null) {
        return principal.getDisplayName() != null ? principal.getDisplayName() : "anonyme";
      }
    }
    return "anonyme";
  }

  private String getIP(ServletRequest request) {
    return request.getRemoteAddr();
  }

  public void init(FilterConfig filterConfig) throws ServletException {
  }

  public void destroy() {
  }
}
  
  
